@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -30,7 +30,7 @@
 #include <QRegExp>
 #include <QDebug>
 
-static const int constSchemaVersion=3;
+static const int constSchemaVersion=5;
 
 bool LibraryDb::dbgEnabled=false;
 #define DBUG if (dbgEnabled) qWarning() << metaObject()->className() << __FUNCTION__ << (void *)this
@@ -45,17 +45,22 @@ LibraryDb::AlbumSort LibraryDb::toAlbumSort(const QString &str)
             return (AlbumSort)i;
         }
     }
-    return AS_Album;
+    return AS_AlArYr;
 }
 
 QString LibraryDb::albumSortStr(AlbumSort m)
 {
     switch(m) {
-    case AS_Artist:   return "artist";
+    case AS_ArAlYr:   return "artist";
     default:
-    case AS_Album:    return "album";
-    case AS_Year:     return "year";
+    case AS_AlArYr:   return "album";
+    case AS_YrAlAr:   return "year";
     case AS_Modified: return "modified";
+
+    // Re-added in 2.1
+    case AS_ArYrAl:   return "aryral";
+    case AS_AlYrAr:   return "alyral";
+    case AS_YrArAl:   return "yraral";
     }
 }
 
@@ -96,41 +101,41 @@ static bool albumsSortArAlYr(const LibraryDb::Album &a, const LibraryDb::Album &
     return a.year<b.year;
 }
 
-//static bool albumsSortAlYrAr(const LibraryDb::Album &a, const LibraryDb::Album &b)
-//{
-//    const QString &an=a.sort.isEmpty() ? a.name : a.sort;
-//    const QString &bn=b.sort.isEmpty() ? b.name : b.sort;
-//    int cmp=an.localeAwareCompare(bn);
-//    if (cmp!=0) {
-//        return cmp<0;
-//    }
+static bool albumsSortAlYrAr(const LibraryDb::Album &a, const LibraryDb::Album &b)
+{
+    const QString &an=a.sort.isEmpty() ? a.name : a.sort;
+    const QString &bn=b.sort.isEmpty() ? b.name : b.sort;
+    int cmp=an.localeAwareCompare(bn);
+    if (cmp!=0) {
+        return cmp<0;
+    }
 
-//    if (a.year!=b.year) {
-//        return a.year<b.year;
-//    }
+    if (a.year!=b.year) {
+        return a.year<b.year;
+    }
 
-//    const QString &aa=a.artistSort.isEmpty() ? a.artist : a.artistSort;
-//    const QString &ba=b.artistSort.isEmpty() ? b.artist : b.artistSort;
-//    return aa.localeAwareCompare(ba)<0;
-//}
+    const QString &aa=a.artistSort.isEmpty() ? a.artist : a.artistSort;
+    const QString &ba=b.artistSort.isEmpty() ? b.artist : b.artistSort;
+    return aa.localeAwareCompare(ba)<0;
+}
 
-//static bool albumsSortArYrAl(const LibraryDb::Album &a, const LibraryDb::Album &b)
-//{
-//    const QString &aa=a.artistSort.isEmpty() ? a.artist : a.artistSort;
-//    const QString &ba=b.artistSort.isEmpty() ? b.artist : b.artistSort;
-//    int cmp=aa.localeAwareCompare(ba);
-//    if (cmp!=0) {
-//        return cmp<0;
-//    }
+static bool albumsSortArYrAl(const LibraryDb::Album &a, const LibraryDb::Album &b)
+{
+    const QString &aa=a.artistSort.isEmpty() ? a.artist : a.artistSort;
+    const QString &ba=b.artistSort.isEmpty() ? b.artist : b.artistSort;
+    int cmp=aa.localeAwareCompare(ba);
+    if (cmp!=0) {
+        return cmp<0;
+    }
 
-//    if (a.year!=b.year) {
-//        return a.year<b.year;
-//    }
+    if (a.year!=b.year) {
+        return a.year<b.year;
+    }
 
-//    const QString &an=a.sort.isEmpty() ? a.name : a.sort;
-//    const QString &bn=b.sort.isEmpty() ? b.name : b.sort;
-//    return an.localeAwareCompare(bn)<0;
-//}
+    const QString &an=a.sort.isEmpty() ? a.name : a.sort;
+    const QString &bn=b.sort.isEmpty() ? b.name : b.sort;
+    return an.localeAwareCompare(bn)<0;
+}
 
 static bool albumsSortYrAlAr(const LibraryDb::Album &a, const LibraryDb::Album &b)
 {
@@ -150,23 +155,23 @@ static bool albumsSortYrAlAr(const LibraryDb::Album &a, const LibraryDb::Album &
     return aa.localeAwareCompare(ba)<0;
 }
 
-//static bool albumsSortYrArAl(const LibraryDb::Album &a, const LibraryDb::Album &b)
-//{
-//    if (a.year!=b.year) {
-//        return a.year<b.year;
-//    }
+static bool albumsSortYrArAl(const LibraryDb::Album &a, const LibraryDb::Album &b)
+{
+    if (a.year!=b.year) {
+        return a.year<b.year;
+    }
 
-//    const QString &aa=a.artistSort.isEmpty() ? a.artist : a.artistSort;
-//    const QString &ba=b.artistSort.isEmpty() ? b.artist : b.artistSort;
-//    int cmp=aa.localeAwareCompare(ba);
-//    if (cmp!=0) {
-//        return cmp<0;
-//    }
+    const QString &aa=a.artistSort.isEmpty() ? a.artist : a.artistSort;
+    const QString &ba=b.artistSort.isEmpty() ? b.artist : b.artistSort;
+    int cmp=aa.localeAwareCompare(ba);
+    if (cmp!=0) {
+        return cmp<0;
+    }
 
-//    const QString &an=a.sort.isEmpty() ? a.name : a.sort;
-//    const QString &bn=b.sort.isEmpty() ? b.name : b.sort;
-//    return an.localeAwareCompare(bn)<0;
-//}
+    const QString &an=a.sort.isEmpty() ? a.name : a.sort;
+    const QString &bn=b.sort.isEmpty() ? b.name : b.sort;
+    return an.localeAwareCompare(bn)<0;
+}
 
 static bool albumsSortModified(const LibraryDb::Album &a, const LibraryDb::Album &b)
 {
@@ -195,7 +200,7 @@ static bool songSort(const Song &a, const Song &b)
     if (0!=cmp) {
         return cmp<0;
     }
-    cmp=a.genre.localeAwareCompare(b.genre);
+    cmp=a.displayGenre().localeAwareCompare(b.displayGenre());
     if (0!=cmp) {
         return cmp<0;
     }
@@ -351,6 +356,10 @@ static bool songsSortArAl(const Song &a, const Song &b)
 
 static QString artistSort(const Song &s)
 {
+    if (s.useComposer() && !s.composer().isEmpty()) {
+        return s.composer();
+
+    }
     if (!s.artistSortString().isEmpty()) {
         return s.artistSortString();
     }
@@ -392,6 +401,17 @@ public:
             // to bound parameters
             if (QVariant::Int==value.type()) {
                 whereClauses << QString("%1 %2 %3").arg(column, op, value.toString());
+            } else if ("genre"==column) {
+                QString clause("(");
+                for (int i=0; i<Song::constNumGenres; ++i) {
+                    clause+=QString("%1 %2 ?").arg(column+QString::number(i+1), op);
+                    if (i<(Song::constNumGenres-1)) {
+                        clause+=" OR ";
+                    }
+                    boundValues << value;
+                }
+                clause+=")";
+                whereClauses << clause;
             } else {
                 whereClauses << QString("%1 %2 ?").arg(column, op);
                 boundValues << value;
@@ -503,11 +523,15 @@ enum SongFields {
     SF_albumId,
     SF_albumSort,
     SF_title,
-    SF_genre,
+    SF_genre1,
+    SF_genre2,
+    SF_genre3,
+    SF_genre4,
     SF_track,
     SF_disc,
     SF_time,
     SF_year,
+    SF_origYear,
     SF_type,
     SF_lastModified
 };
@@ -525,6 +549,10 @@ bool LibraryDb::init(const QString &dbFile)
     DBUG << dbFile << dbName;
     currentVersion=0;
     db=new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE", dbName.isEmpty() ? QLatin1String(QSqlDatabase::defaultConnection) : dbName));
+    if (!db || !db->isValid()) {
+        emit error(tr("Database error - please check Qt SQLite driver is installed"));
+        return false;
+    }
     db->setDatabaseName(dbFile);
     DBUG << (void *)db;
     if (!db->open()) {
@@ -568,15 +596,18 @@ bool LibraryDb::init(const QString &dbFile)
                     "albumId text, "
                     "albumSort text, "
                     "title text, "
-                    "genre text, "
+                    "genre1 text, "
+                    "genre2 text, "
+                    "genre3 text, "
+                    "genre4 text, "
                     "track integer, "
                     "disc integer, "
                     "time integer, "
                     "year integer, "
+                    "origYear integer, "
                     "type integer, "
                     "lastModified integer, "
                     "primary key (file))")) {
-        #ifndef CANTATA_WEB
         QSqlQuery fts(*db);
         if (!fts.exec("create virtual table if not exists songs_fts using fts4(fts_artist, fts_artistId, fts_album, fts_albumId, fts_title, tokenize=unicode61)")) {
             DBUG << "Failed to create FTS table" << fts.lastError().text() << "trying again with simple tokenizer";
@@ -584,7 +615,6 @@ bool LibraryDb::init(const QString &dbFile)
                 DBUG << "Failed to create FTS table" << fts.lastError().text();
             }
         }
-        #endif
     } else {
         DBUG << "Failed to create songs table";
         return false;
@@ -598,8 +628,8 @@ void LibraryDb::insertSong(const Song &s)
 {
     if (!insertSongQuery) {
         insertSongQuery=new QSqlQuery(*db);
-        insertSongQuery->prepare("insert into songs(file, artist, artistId, albumArtist, artistSort, composer, album, albumId, albumSort, title, genre, track, disc, time, year, type, lastModified) "
-                                 "values(:file, :artist, :artistId, :albumArtist, :artistSort, :composer, :album, :albumId, :albumSort, :title, :genre, :track, :disc, :time, :year, :type, :lastModified)");
+        insertSongQuery->prepare("insert into songs(file, artist, artistId, albumArtist, artistSort, composer, album, albumId, albumSort, title, genre1, genre2, genre3, genre4, track, disc, time, year, origYear, type, lastModified) "
+                                 "values(:file, :artist, :artistId, :albumArtist, :artistSort, :composer, :album, :albumId, :albumSort, :title, :genre1, :genre2, :genre3, :genre4, :track, :disc, :time, :year, :origYear, :type, :lastModified)");
     }
     QString albumId=s.albumId();
     insertSongQuery->bindValue(":file", s.file);
@@ -612,11 +642,14 @@ void LibraryDb::insertSong(const Song &s)
     insertSongQuery->bindValue(":albumId", albumId);
     insertSongQuery->bindValue(":albumSort", albumSort(s));
     insertSongQuery->bindValue(":title", s.title);
-    insertSongQuery->bindValue(":genre", s.genre.isEmpty() ? constNullGenre : s.genre);
+    for (int i=0; i<Song::constNumGenres; ++i) {
+        insertSongQuery->bindValue(":genre"+QString::number(i+1), s.genres[i].isEmpty() ? constNullGenre : s.genres[i]);
+    }
     insertSongQuery->bindValue(":track", s.track);
     insertSongQuery->bindValue(":disc", s.disc);
     insertSongQuery->bindValue(":time", s.time);
     insertSongQuery->bindValue(":year", s.year);
+    insertSongQuery->bindValue(":origYear", s.origYear);
     insertSongQuery->bindValue(":type", s.type);
     insertSongQuery->bindValue(":lastModified", s.lastModified);
     if (!insertSongQuery->exec()) {
@@ -627,24 +660,34 @@ void LibraryDb::insertSong(const Song &s)
 QList<LibraryDb::Genre> LibraryDb::getGenres()
 {
     DBUG;
-    QMap<QString, int> map;
+    QMap<QString, QSet<QString> > map;
     if (0!=currentVersion && db) {
-        SqlQuery query("distinct genre, artistId", *db);
+        QString queryStr("distinct ");
+        for (int i=0; i<Song::constNumGenres; ++i) {
+            queryStr+="genre"+QString::number(i+1)+", ";
+        }
+        queryStr+="artistId";
+        SqlQuery query(queryStr, *db);
         query.setFilter(filter);
 
         query.exec();
         DBUG << query.executedQuery();
         while (query.next()) {
-            map[query.value(0).toString()]++;
+            for (int i=0; i<Song::constNumGenres; ++i) {
+                QString genre=query.value(i).toString();
+                if (!genre.isEmpty() && genre!=constNullGenre) {
+                    map[genre].insert(query.value(Song::constNumGenres).toString());
+                }
+            }
         }
     }
 
     QList<LibraryDb::Genre> genres;
-    QMap<QString, int>::ConstIterator it=map.constBegin();
-    QMap<QString, int>::ConstIterator end=map.constEnd();
+    QMap<QString, QSet<QString> >::ConstIterator it=map.constBegin();
+    QMap<QString, QSet<QString> >::ConstIterator end=map.constEnd();
     for (; it!=end; ++it) {
         DBUG << it.key();
-        genres.append(Genre(it.key(), it.value()));
+        genres.append(Genre(it.key(), it.value().size()));
     }
     qSort(genres);
     return genres;
@@ -660,6 +703,8 @@ QList<LibraryDb::Artist> LibraryDb::getArtists(const QString &genre)
         query.setFilter(filter);
         if (!genre.isEmpty()) {
             query.addWhere("genre", genre);
+        } else if (!genreFilter.isEmpty()) {
+            query.addWhere("genre", genreFilter);
         }
         query.exec();
         DBUG << query.executedQuery();
@@ -689,34 +734,65 @@ QList<LibraryDb::Album> LibraryDb::getAlbums(const QString &artistId, const QStr
     if (0!=currentVersion && db) {
         bool wantModified=AS_Modified==sort;
         bool wantArtist=artistId.isEmpty();
-        int artistCol=wantModified ? 6 : 5;
-        SqlQuery query(QLatin1String("album, albumId, albumSort, year, time")+(wantModified ? ", lastModified" : "")+(wantArtist ? ", artistId, artistSort" : ""), *db);
+        QString queryString="album, albumId, albumSort, artist, albumArtist, composer";
+        for (int i=0; i<Song::constNumGenres; ++i) {
+            queryString+=", genre"+QString::number(i+1);
+        }
+        queryString+=", year, time";
+        if (wantModified) {
+            queryString+=", lastModified";
+        }
+        if (wantArtist) {
+            queryString+=", artistId, artistSort";
+        }
+        SqlQuery query(queryString, *db);
         query.setFilter(filter);
         if (!artistId.isEmpty()) {
             query.addWhere("artistId", artistId);
         }
         if (!genre.isEmpty()) {
             query.addWhere("genre", genre);
+        } else if (!genreFilter.isEmpty()) {
+            query.addWhere("genre", genreFilter);
         }
         query.exec();
         DBUG << query.executedQuery() << timer.elapsed();
         int count=0;
         QMap<QString, Album> entries;
+        QMap<QString, QSet<QString> > albumIdArtists; // MAp of albumId -> albumartists/composers
         while (query.next()) {
             count++;
-            QString album=query.value(0).toString();
-            QString albumId=query.value(1).toString();
-            QString albumSort=query.value(2).toString();
-            int year=query.value(3).toInt();
-            int time=query.value(4).toInt();
-            int lastModified=wantModified ? query.value(5).toInt() : 0;
-            QString artist=wantArtist ? query.value(artistCol).toString() : QString();
-            QString artistSort=wantArtist ? query.value(artistCol+1).toString() : QString();
-            QString key='{'+albumId+"}{"+(wantArtist ? artist : artistId)+'}';
+            int col=0;
+            QString album=query.value(col++).toString();
+            QString albumId=query.value(col++).toString();
+            QString albumSort=query.value(col++).toString();
+
+            Song s;
+            s.artist=query.value(col++).toString();
+            s.albumartist=query.value(col++).toString();
+            s.setComposer(query.value(col++).toString());
+            s.album=album.isEmpty() ? albumId : album;
+            for (int i=0; i<Song::constNumGenres; ++i) {
+                QString genre=query.value(col++).toString();
+                if (genre!=constNullGenre) {
+                    s.addGenre(genre);
+                }
+            }
+            album=s.displayAlbum();
+            int year=query.value(col++).toInt();
+            int time=query.value(col++).toInt();
+            int lastModified=wantModified ? query.value(col++).toInt() : 0;
+            QString artist=wantArtist ? query.value(col++).toString() : QString();
+            QString artistSort=wantArtist ? query.value(col++).toString() : QString();
+            // If listing albums not filtered on artist, then if we have a unqique id for the album use that.
+            // This will allow us to grouup albums with different composers when the composer tweak is set
+            // Issue #1025
+            bool haveUniqueId=wantArtist && !albumId.isEmpty() && !album.isEmpty() && albumId!=album;
+            QString key=haveUniqueId ? albumId : ('{'+albumId+"}{"+(wantArtist ? artist : artistId)+'}');
             QMap<QString, Album>::iterator it=entries.find(key);
 
             if (it==entries.end()) {
-                entries.insert(key, Album(album.isEmpty() ? albumId : album, albumId, albumSort, artist, artistSort, year, 1, time, lastModified));
+                entries.insert(key, Album(album.isEmpty() ? albumId : album, albumId, albumSort, artist, artistSort, year, 1, time, lastModified, haveUniqueId));
             } else {
                 Album &al=it.value();
                 if (wantModified) {
@@ -726,22 +802,52 @@ QList<LibraryDb::Album> LibraryDb::getAlbums(const QString &artistId, const QStr
                 al.duration+=time;
                 al.trackCount++;
             }
+            if (haveUniqueId) {
+                QMap<QString, QSet<QString> >::iterator aIt = albumIdArtists.find(key);
+                if (aIt == albumIdArtists.end()) {
+                    albumIdArtists.insert(key, QSet<QString>() << artist);
+                } else {
+                    aIt.value().insert(artist);
+                }
+            }
         }
+
+        QMap<QString, QSet<QString> >::ConstIterator aIt = albumIdArtists.constBegin();
+        QMap<QString, QSet<QString> >::ConstIterator aEnd = albumIdArtists.constEnd();
+        for(; aIt!=aEnd; ++aIt) {
+            if (aIt.value().count()>1) {
+                QStringList artists = aIt.value().toList();
+                artists.sort();
+                Album &al = entries.find(aIt.key()).value();
+                al.artist = artists.join(", ");
+                al.artistSort = QString();
+            }
+        }
+
         albums=entries.values();
         DBUG << count << albums.count();
     }
 
     DBUG << "After select" << timer.elapsed();
     switch(sort) {
-    case AS_Album:
+    case AS_AlArYr:
         qSort(albums.begin(), albums.end(), albumsSortAlArYr);
         break;
-    case AS_Artist:
+    case AS_AlYrAr:
+        qSort(albums.begin(), albums.end(), albumsSortAlYrAr);
+        break;
+    case AS_ArAlYr:
         qSort(albums.begin(), albums.end(), albumsSortArAlYr);
         break;
-    case AS_Year:
+    case AS_ArYrAl:
+        qSort(albums.begin(), albums.end(), albumsSortArYrAl);
+        break;
+    case AS_YrAlAr:
         qSort(albums.begin(), albums.end(), albumsSortYrAlAr);
         break;
+    case AS_YrArAl:
+        qSort(albums.begin(), albums.end(), albumsSortYrArAl);
+        break;
     case AS_Modified:
         qSort(albums.begin(), albums.end(), albumsSortModified);
         break;
@@ -769,6 +875,8 @@ QList<Song> LibraryDb::getTracks(const QString &artistId, const QString &albumId
         }
         if (!genre.isEmpty()) {
             query.addWhere("genre", genre);
+        } else if (!genreFilter.isEmpty()) {
+            query.addWhere("genre", genreFilter);
         }
         query.exec();
         DBUG << query.executedQuery();
@@ -778,10 +886,10 @@ QList<Song> LibraryDb::getTracks(const QString &artistId, const QString &albumId
     }
 
     switch(sort) {
-    case AS_Album:
+    case AS_AlArYr:
         qSort(songs.begin(), songs.end(), songsSortAlAr);
         break;
-    case AS_Artist:
+    case AS_ArAlYr:
         qSort(songs.begin(), songs.end(), songsSortArAl);
         break;
 //    case AS_Year:
@@ -799,20 +907,25 @@ QList<Song> LibraryDb::getTracks(const QString &artistId, const QString &albumId
 QList<Song> LibraryDb::getTracks(int rowFrom, int count)
 {
     QList<Song> songList;
-    SqlQuery query("*", *db);
-    query.addWhere("rowid", rowFrom, ">");
-    query.addWhere("rowid", rowFrom+count, "<=");
-    query.addWhere("type", 0);
-    query.exec();
-    DBUG << query.executedQuery();
-    while (query.next()) {
-        songList.append(getSong(query.realQuery()));
+    if (db) {
+        SqlQuery query("*", *db);
+        query.addWhere("rowid", rowFrom, ">");
+        query.addWhere("rowid", rowFrom+count, "<=");
+        query.addWhere("type", 0);
+        query.exec();
+        DBUG << query.executedQuery();
+        while (query.next()) {
+            songList.append(getSong(query.realQuery()));
+        }
     }
     return songList;
 }
 
 int LibraryDb::trackCount()
 {
+    if (!db) {
+        return 0;
+    }
     SqlQuery query("(count())", *db);
     query.addWhere("type", 0);
     query.exec();
@@ -820,19 +933,20 @@ int LibraryDb::trackCount()
     return query.next() ? query.value(0).toInt() : 0;
 }
 
-#ifndef CANTATA_WEB
 QList<Song> LibraryDb::songs(const QStringList &files, bool allowPlaylists) const
 {
     QList<Song> songList;
-    foreach (const QString &f, files) {
-        SqlQuery query("*", *db);
-        query.addWhere("file", f);
-        query.exec();
-        DBUG << query.executedQuery();
-        if (query.next()) {
-            Song song=getSong(query.realQuery());
-            if (allowPlaylists || Song::Playlist!=song.type) {
-                songList.append(song);
+    if (0!=currentVersion && db) {
+        foreach (const QString &f, files) {
+            SqlQuery query("*", *db);
+            query.addWhere("file", f);
+            query.exec();
+            DBUG << query.executedQuery();
+            if (query.next()) {
+                Song song=getSong(query.realQuery());
+                if (allowPlaylists || Song::Playlist!=song.type) {
+                    songList.append(song);
+                }
             }
         }
     }
@@ -872,6 +986,8 @@ LibraryDb::Album LibraryDb::getRandomAlbum(const QString &genre, const QString &
         }
         if (!genre.isEmpty()) {
             query.addWhere("genre", genre);
+        } else if (!genreFilter.isEmpty()) {
+            query.addWhere("genre", genreFilter);
         }
         query.exec();
         DBUG << query.executedQuery();
@@ -912,13 +1028,30 @@ QSet<QString> LibraryDb::get(const QString &type)
         return detailsCache[type];
     }
     QSet<QString> set;
-    SqlQuery query("distinct "+type, *db);
-    query.exec();
-    DBUG << query.executedQuery();
-    while (query.next()) {
-        QString val=query.value(0).toString();
-        if (!val.isEmpty()) {
-            set.insert(val);
+    if (!db) {
+        return set;
+    }
+
+    QStringList columns;
+    bool isGenre="genre"==type;
+
+    if (isGenre) {
+        for (int i=0; i<Song::constNumGenres; ++i) {
+            columns << type+QString::number(i+1);
+        }
+    } else {
+        columns << type;
+    }
+
+    foreach (const QString &col, columns) {
+        SqlQuery query("distinct "+col, *db);
+        query.exec();
+        DBUG << query.executedQuery();
+        while (query.next()) {
+            QString val=query.value(0).toString();
+            if (!val.isEmpty() && (!isGenre || constNullGenre!=val)) {
+                set.insert(val);
+            }
         }
     }
     detailsCache[type]=set;
@@ -936,6 +1069,9 @@ void LibraryDb::getDetails(QSet<QString> &artists, QSet<QString> &albumArtists,
 
 bool LibraryDb::songExists(const Song &song)
 {
+    if (!db) {
+        return false;
+    }
     SqlQuery query("file", *db);
     query.addWhere("artistId", song.artistOrComposer());
     query.addWhere("albumId", song.albumId());
@@ -945,7 +1081,7 @@ bool LibraryDb::songExists(const Song &song)
     return query.next();
 }
 
-bool LibraryDb::setFilter(const QString &f)
+bool LibraryDb::setFilter(const QString &f, const QString &genre)
 {
     QString newFilter=f.trimmed().toLower();
 
@@ -965,13 +1101,11 @@ bool LibraryDb::setFilter(const QString &f)
         newFilter=tokens.join(" ");
         DBUG << newFilter;
     }
-    if (newFilter!=filter) {
-        filter=newFilter;
-        return true;
-    }
-    return false;
+    bool modified=newFilter!=filter || genre!=genreFilter;
+    filter=newFilter;
+    genreFilter=genre;
+    return modified;
 }
-#endif
 
 void LibraryDb::updateStarted(time_t ver)
 {
@@ -1006,11 +1140,9 @@ void LibraryDb::updateFinished()
         return;
     }
     DBUG << timer.elapsed();
-    #ifndef CANTATA_WEB
     DBUG << "update fts" << timer.elapsed();
     QSqlQuery(*db).exec("insert into songs_fts(fts_artist, fts_artistId, fts_album, fts_albumId, fts_title) "
                         "select artist, artistId, album, albumId, title from songs");
-    #endif
     QSqlQuery(*db).exec("update versions set collection ="+QString::number(newVersion));
     DBUG << "commit" << timer.elapsed();
     db->commit();
@@ -1021,11 +1153,16 @@ void LibraryDb::updateFinished()
 
 void LibraryDb::abortUpdate()
 {
-    db->rollback();
+    if (db) {
+        db->rollback();
+    }
 }
 
 bool LibraryDb::createTable(const QString &q)
 {
+    if (!db) {
+        return false;
+    }
     QSqlQuery query(*db);
     if (!query.exec("create table if not exists "+q)) {
         qWarning() << "Failed to create table" << query.lastError().text();
@@ -1051,11 +1188,17 @@ Song LibraryDb::getSong(const QSqlQuery &query)
         s.setMbAlbumId(val);
     }
     s.title=query.value(SF_title).toString();
-    s.genre=query.value(SF_genre).toString();
+    for (int i=SF_genre1; i<SF_genre1+Song::constNumGenres; ++i) {
+        QString genre=query.value(i).toString();
+        if (genre!=constNullGenre) {
+            s.addGenre(genre);
+        }
+    }
     s.track=query.value(SF_track).toUInt();
     s.disc=query.value(SF_disc).toUInt();
     s.time=query.value(SF_time).toUInt();
     s.year=query.value(SF_year).toUInt();
+    s.origYear=query.value(SF_origYear).toUInt();
     s.type=(Song::Type)query.value(SF_type).toUInt();
     val=query.value(SF_artistSort).toString();
     if (!val.isEmpty() && val!=s.albumArtist()) {
@@ -1092,10 +1235,8 @@ void LibraryDb::clearSongs(bool startTransaction)
         db->transaction();
     }
     QSqlQuery(*db).exec("delete from songs");
-    #ifndef CANTATA_WEB
     QSqlQuery(*db).exec("delete from songs_fts");
     detailsCache.clear();
-    #endif
     if (startTransaction) {
         db->commit();
     }
