@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,16 +29,16 @@
 #include "stdactions.h"
 #include "customactions.h"
 #include "support/utils.h"
-#include "support/localize.h"
 #include "support/messagebox.h"
 #include "support/actioncollection.h"
-//#include "support/combobox.h"
 #include "models/mpdlibrarymodel.h"
 #include "widgets/menubutton.h"
+#include "widgets/genrecombo.h"
 
 LibraryPage::LibraryPage(QWidget *p)
     : SinglePageWidget(p)
 {
+    genreCombo=new GenreCombo(this);
     connect(StdActions::self()->addRandomAlbumToPlayQueueAction, SIGNAL(triggered()), SLOT(addRandomAlbum()));
     connect(MPDConnection::self(), SIGNAL(updatingLibrary(time_t)), view, SLOT(updating()));
     connect(MPDConnection::self(), SIGNAL(updatedLibrary()), view, SLOT(updated()));
@@ -47,35 +47,28 @@ LibraryPage::LibraryPage(QWidget *p)
     connect(view, SIGNAL(itemsSelected(bool)), this, SLOT(controlActions()));
     connect(view, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(itemDoubleClicked(const QModelIndex &)));
     view->setModel(MpdLibraryModel::self());
-
-//    groupCombo=new ComboBox(this);
-//    groupCombo->addItem(i18n("Genre"), SqlLibraryModel::T_Genre);
-//    groupCombo->addItem(i18n("Atists"), SqlLibraryModel::T_Artist);
-//    groupCombo->addItem(i18n("Albums"), SqlLibraryModel::T_Album);
+    connect(MpdLibraryModel::self(), SIGNAL(modelReset()), this, SLOT(modelReset()));
 
     // Settings...
     Configuration config(metaObject()->className());
     view->setMode(ItemView::Mode_DetailedTree);
     MpdLibraryModel::self()->load(config);
-//    for (int i=0; i<groupCombo->count(); ++i) {
-//        if (groupCombo->itemData(i).toInt()==MpdLibraryModel::self()->topLevel()) {
-//            groupCombo->setCurrentIndex(i);
-//            break;
-//        }
-//    }
 
     config.beginGroup(SqlLibraryModel::groupingStr(MpdLibraryModel::self()->topLevel()));
     view->load(config);
 
-    showArtistImagesAction=new QAction(i18n("Show Artist Images"), this);
+    showArtistImagesAction=new QAction(tr("Show Artist Images"), this);
     showArtistImagesAction->setCheckable(true);
-    libraryAlbumSortAction=createMenuGroup(i18n("Sort Albums"), QList<MenuItem>() << MenuItem(i18n("Name"), LibraryDb::AS_Album)
-                                                                                  << MenuItem(i18n("Year"), LibraryDb::AS_Year),
+    libraryAlbumSortAction=createMenuGroup(tr("Sort Albums"), QList<MenuItem>() << MenuItem(tr("Name"), LibraryDb::AS_AlArYr)
+                                                                                  << MenuItem(tr("Year"), LibraryDb::AS_YrAlAr),
                                            MpdLibraryModel::self()->libraryAlbumSort(), this, SLOT(libraryAlbumSortChanged()));
-    albumAlbumSortAction=createMenuGroup(i18n("Sort Albums"), QList<MenuItem>() << MenuItem(i18n("Name"), LibraryDb::AS_Album)
-                                                                                << MenuItem(i18n("Artist"), LibraryDb::AS_Artist)
-                                                                                << MenuItem(i18n("Year"), LibraryDb::AS_Year)
-                                                                                << MenuItem(i18n("Modified Date"), LibraryDb::AS_Modified),
+    albumAlbumSortAction=createMenuGroup(tr("Sort Albums"), QList<MenuItem>() << MenuItem(tr("Album, Artist, Year"), LibraryDb::AS_AlArYr)
+                                                                                << MenuItem(tr("Album, Year, Artist"), LibraryDb::AS_AlYrAr)
+                                                                                << MenuItem(tr("Artist, Album, Year"), LibraryDb::AS_ArAlYr)
+                                                                                << MenuItem(tr("Artist, Year, Album"), LibraryDb::AS_ArYrAl)
+                                                                                << MenuItem(tr("Year, Album, Artist"), LibraryDb::AS_YrAlAr)
+                                                                                << MenuItem(tr("Year, Artist, Album"), LibraryDb::AS_YrArAl)
+                                                                                << MenuItem(tr("Modified Date"), LibraryDb::AS_Modified),
                                          MpdLibraryModel::self()->albumAlbumSort(), this, SLOT(albumAlbumSortChanged()));
 
     MenuButton *menu=new MenuButton(this);
@@ -84,10 +77,11 @@ LibraryPage::LibraryPage(QWidget *p)
                               << ItemView::Mode_IconTop);
     menu->addAction(viewAction);
 
-    menu->addAction(createMenuGroup(i18n("Group By"), QList<MenuItem>() << MenuItem(i18n("Genre"), SqlLibraryModel::T_Genre)
-                                                                        << MenuItem(i18n("Artist"), SqlLibraryModel::T_Artist)
-                                                                        << MenuItem(i18n("Album"), SqlLibraryModel::T_Album),
+    menu->addAction(createMenuGroup(tr("Group By"), QList<MenuItem>() << MenuItem(tr("Genre"), SqlLibraryModel::T_Genre)
+                                                                        << MenuItem(tr("Artist"), SqlLibraryModel::T_Artist)
+                                                                        << MenuItem(tr("Album"), SqlLibraryModel::T_Album),
                                     MpdLibraryModel::self()->topLevel(), this, SLOT(groupByChanged())));
+    genreCombo->setVisible(SqlLibraryModel::T_Genre!=MpdLibraryModel::self()->topLevel());
 
     menu->addAction(libraryAlbumSortAction);
     menu->addAction(albumAlbumSortAction);
@@ -97,8 +91,9 @@ LibraryPage::LibraryPage(QWidget *p)
     albumAlbumSortAction->setVisible(SqlLibraryModel::T_Album==MpdLibraryModel::self()->topLevel());
     libraryAlbumSortAction->setVisible(SqlLibraryModel::T_Album!=MpdLibraryModel::self()->topLevel());
     showArtistImagesAction->setVisible(SqlLibraryModel::T_Album!=MpdLibraryModel::self()->topLevel());
-    init(ReplacePlayQueue|AppendToPlayQueue, QList<QWidget *>() << menu/* << groupCombo*/);
-//    connect(groupCombo, SIGNAL(activated(int)), SLOT(groupByChanged()));
+    genreCombo->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
+    init(ReplacePlayQueue|AppendToPlayQueue, QList<QWidget *>() << menu << genreCombo);
+    connect(genreCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(doSearch()));
     view->addAction(StdActions::self()->addToStoredPlaylistAction);
     view->addAction(CustomActions::self());
     #ifdef TAGLIB_FOUND
@@ -191,8 +186,8 @@ void LibraryPage::deleteSongs()
     QList<Song> songs=selectedSongs();
 
     if (!songs.isEmpty()) {
-        if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to delete the selected songs?\n\nThis cannot be undone."),
-                                                      i18n("Delete Songs"), StdGuiItem::del(), StdGuiItem::cancel())) {
+        if (MessageBox::Yes==MessageBox::warningYesNo(this, tr("Are you sure you wish to delete the selected songs?\n\nThis cannot be undone."),
+                                                      tr("Delete Songs"), StdGuiItem::del(), StdGuiItem::cancel())) {
             emit deleteSongs(QString(), songs);
         }
         view->clearSelection();
@@ -276,6 +271,11 @@ void LibraryPage::setView(int v)
     showArtistImagesAction->setVisible(SqlLibraryModel::T_Album!=MpdLibraryModel::self()->topLevel() && ItemView::Mode_IconTop!=view->viewMode());
 }
 
+void LibraryPage::modelReset()
+{
+    genreCombo->update(MpdLibraryModel::self()->getGenres());
+}
+
 void LibraryPage::groupByChanged()
 {
     QAction *act=qobject_cast<QAction *>(sender());
@@ -283,7 +283,6 @@ void LibraryPage::groupByChanged()
         return;
     }
     int mode=act->property(constValProp).toInt();
-//    int mode=groupCombo->itemData(groupCombo->currentIndex()).toInt();
 
     Configuration config(metaObject()->className());
     config.beginGroup(SqlLibraryModel::groupingStr(MpdLibraryModel::self()->topLevel()));
@@ -293,6 +292,7 @@ void LibraryPage::groupByChanged()
     albumAlbumSortAction->setVisible(SqlLibraryModel::T_Album==MpdLibraryModel::self()->topLevel());
     libraryAlbumSortAction->setVisible(SqlLibraryModel::T_Album!=MpdLibraryModel::self()->topLevel());
     showArtistImagesAction->setVisible(SqlLibraryModel::T_Album!=MpdLibraryModel::self()->topLevel() && ItemView::Mode_IconTop!=view->viewMode());
+    genreCombo->setVisible(SqlLibraryModel::T_Genre!=MpdLibraryModel::self()->topLevel());
 
     config.endGroup();
     config.beginGroup(SqlLibraryModel::groupingStr(MpdLibraryModel::self()->topLevel()));
@@ -410,7 +410,8 @@ void LibraryPage::addRandomAlbum()
 
 void LibraryPage::doSearch()
 {
-    MpdLibraryModel::self()->search(view->searchText());
+    MpdLibraryModel::self()->search(view->searchText(),
+                                    genreCombo->isHidden() || genreCombo->currentIndex()<=0 ? QString() : genreCombo->currentText());
 }
 
 void LibraryPage::controlActions()
