@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -170,6 +170,7 @@ MPDConnectionDetails Settings::connectionDetails(const QString &name)
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         details.streamUrl=grp.get("streamUrl", QString());
         #endif
+        details.replayGain=grp.get("replayGain", QString());
     } else {
         details.hostname=mpdDefaults.host;
         details.port=mpdDefaults.port;
@@ -186,11 +187,7 @@ MPDConnectionDetails Settings::connectionDetails(const QString &name)
 
 QList<MPDConnectionDetails> Settings::allConnections()
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    QStringList groups=KGlobal::config()->groupList();
-    #else
     QStringList groups=cfg.childGroups();
-    #endif
     QList<MPDConnectionDetails> connections;
     foreach (const QString &grp, groups) {
         if (cfg.hasGroup(grp) && grp.startsWith("Connection")) {
@@ -449,15 +446,6 @@ QString Settings::httpInterface()
     return cfg.get("httpInterface", QString());
 }
 
-bool Settings::alwaysUseHttp()
-{
-    #ifdef ENABLE_HTTP_SERVER
-    return cfg.get("alwaysUseHttp", false);
-    #else
-    return false;
-    #endif
-}
-
 int Settings::playQueueView()
 {
     if (version()<CANTATA_MAKE_VERSION(1, 3, 53)) {
@@ -614,21 +602,6 @@ int Settings::podcastAutoDownloadLimit()
     return cfg.get("podcastAutoDownloadLimit", 0, 0, 1000);
 }
 
-int Settings::maxCoverUpdatePerIteration()
-{
-    return cfg.get("maxCoverUpdatePerIteration", 10, 1, 50);
-}
-
-int Settings::coverCacheSize()
-{
-    return cfg.get("coverCacheSize", 10, 1, 512);
-}
-
-QStringList Settings::cueFileCodecs()
-{
-    return cfg.get("cueFileCodecs", QStringList());
-}
-
 bool Settings::networkAccessEnabled()
 {
     return cfg.get("networkAccessEnabled", true);
@@ -644,66 +617,26 @@ Settings::StartupState Settings::startupState()
     return getStartupState(cfg.get("startupState", getStartupStateStr(SS_Previous)));
 }
 
-int Settings::undoSteps()
-{
-    return cfg.get("undoSteps", 10, 0, 20);
-}
-
 QString Settings::searchCategory()
 {
     return cfg.get("searchCategory", QString());
 }
 
-bool Settings::cacheScaledCovers()
-{
-    return cfg.get("cacheScaledCovers", true);
-}
-
 bool Settings::fetchCovers()
 {
     return cfg.get("fetchCovers", true);
 }
 
-int Settings::mpdPoll()
-{
-    return cfg.get("mpdPoll", 0, 0, 60);
-}
-
-
-#ifndef ENABLE_KDE_SUPPORT
 QString Settings::lang()
 {
     return cfg.get("lang", QString());
 }
-#endif
 
 bool Settings::showMenubar()
 {
     return cfg.get("showMenubar", false);
 }
 
-int Settings::menu()
-{
-    #if defined Q_OS_WIN
-    int def=MC_Button|MC_Bar;
-    #elif defined Q_OS_MAC
-    int def=MC_Bar;
-    #else
-    int def=Utils::Gnome==Utils::currentDe() ? MC_Button : Utils::Unity==Utils::currentDe() ? MC_Bar : (MC_Bar|MC_Button);
-    #endif
-    int v=cfg.get("menu", def)&(MC_Bar|MC_Button);
-    return 0==v ? MC_Bar : v;
-}
-
-bool Settings::touchFriendly()
-{
-    #ifdef ENABLE_TOUCH_SUPPORT
-    return cfg.get("touchFriendly", false);
-    #else
-    return false;
-    #endif
-}
-
 bool Settings::showCoverWidget()
 {
     return cfg.get("showCoverWidget", true);
@@ -724,11 +657,6 @@ bool Settings::infoTooltips()
     return cfg.get("infoTooltips", true);
 }
 
-bool Settings::retinaSupport()
-{
-    return cfg.get("retinaSupport", false);
-}
-
 QSet<QString> Settings::ignorePrefixes()
 {
     return cfg.get("ignorePrefixes", Song::ignorePrefixes().toList()).toSet();
@@ -739,6 +667,11 @@ bool Settings::mpris()
     return cfg.get("mpris", true);
 }
 
+QString Settings::style()
+{
+    return cfg.get("style", QString());
+}
+
 void Settings::removeConnectionDetails(const QString &v)
 {
     if (v==currentConnection()) {
@@ -883,11 +816,6 @@ void Settings::saveWikipediaIntroOnly(bool v)
 
 void Settings::saveContextBackdrop(int v)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    if (cfg.get("contextBackdrop", QString()).length()>1) {
-        cfg.removeEntry("contextBackdrop");
-    }
-    #endif
     cfg.set("contextBackdrop", v);
 }
 
@@ -1005,11 +933,6 @@ void Settings::savePlayQueueScroll(bool v)
 
 void Settings::savePlayQueueBackground(int v)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    if (cfg.get("playQueueBackground", QString()).length()>1) {
-        cfg.removeEntry("playQueueBackground");
-    }
-    #endif
     cfg.set("playQueueBackground", v);
 }
 
@@ -1048,11 +971,6 @@ void Settings::savePlayStream(bool v)
 {
     cfg.set("playStream", v);
 }
-
-bool Settings::stopHttpStreamOnPause()
-{
-    return cfg.get("stopHttpStreamOnPause", true);
-}
 #endif
 
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
@@ -1161,27 +1079,16 @@ void Settings::saveFetchCovers(bool v)
     cfg.set("fetchCovers", v);
 }
 
-#ifndef ENABLE_KDE_SUPPORT
 void Settings::saveLang(const QString &v)
 {
     cfg.set("lang", v);
 }
-#endif
 
 void Settings::saveShowMenubar(bool v)
 {
     cfg.set("showMenubar", v);
 }
 
-void Settings::saveTouchFriendly(bool v)
-{
-    #ifdef ENABLE_TOUCH_SUPPORT
-    cfg.set("touchFriendly", v);
-    #else
-    Q_UNUSED(v)
-    #endif
-}
-
 void Settings::saveShowCoverWidget(bool v)
 {
     cfg.set("showCoverWidget", v);
@@ -1202,11 +1109,6 @@ void Settings::saveInfoTooltips(bool v)
     cfg.set("infoTooltips", v);
 }
 
-void Settings::saveRetinaSupport(bool v)
-{
-    cfg.set("retinaSupport", v);
-}
-
 void Settings::saveIgnorePrefixes(const QSet<QString> &v)
 {
     cfg.set("ignorePrefixes", v.toList());
@@ -1217,6 +1119,13 @@ void Settings::saveMpris(bool v)
     cfg.set("mpris", v);
 }
 
+void Settings::saveReplayGain(const QString &conn, const QString &v)
+{
+    QString n=MPDConnectionDetails::configGroupName(conn);
+    Configuration grp(n);
+    grp.set("replayGain", v);
+}
+
 void Settings::save()
 {
     if (AP_NotConfigured!=state) {
