@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -33,11 +33,10 @@
 #include "playqueuemodel.h"
 #include "widgets/groupedview.h"
 #include "roles.h"
-#include "gui/plurals.h"
 #include "gui/covers.h"
 #include "gui/settings.h"
-#include "support/localize.h"
 #include "support/utils.h"
+#include "support/monoicon.h"
 #include "support/globalstatic.h"
 #include "mpd-interface/mpdparseutils.h"
 #include "mpd-interface/mpdstats.h"
@@ -49,11 +48,6 @@
 #include "http/httpserver.h"
 #endif
 
-#if defined ENABLE_MODEL_TEST
-#include "modeltest.h"
-#endif
-
-#ifndef ENABLE_UBUNTU
 QString PlaylistsModel::headerText(int col)
 {
     switch (col) {
@@ -68,19 +62,16 @@ QString PlaylistsModel::headerText(int col)
     default:            return QString();
     }
 }
-#endif
 
 GLOBAL_STATIC(PlaylistsModel, instance)
 
 PlaylistsModel::PlaylistsModel(QObject *parent)
     : ActionModel(parent)
     , multiCol(false)
-    #ifndef ENABLE_UBUNTU
     , itemMenu(0)
     , dropAdjust(0)
-    #endif
 {
-    icn=Icons::self()->playlistFileIcon;
+    icn.addFile(":playlist.svg");
     connect(MPDConnection::self(), SIGNAL(stateChanged(bool)), SLOT(mpdConnectionStateChanged(bool)));
     connect(MPDConnection::self(), SIGNAL(playlistsRetrieved(const QList<Playlist> &)), this, SLOT(setPlaylists(const QList<Playlist> &)));
     connect(MPDConnection::self(), SIGNAL(playlistInfoRetrieved(const QString &, const QList<Song> &)), this, SLOT(playlistInfoRetrieved(const QString &, const QList<Song> &)));
@@ -95,27 +86,20 @@ PlaylistsModel::PlaylistsModel(QObject *parent)
     connect(this, SIGNAL(addToPlaylist(const QString &, const QStringList, quint32, quint32)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList, quint32, quint32)));
     connect(this, SIGNAL(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)), MPDConnection::self(), SLOT(moveInPlaylist(const QString &, const
     QList<quint32> &, quint32, quint32)));
-    #ifndef ENABLE_UBUNTU
     connect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
-    newAction=new QAction(Icon("document-new"), i18n("New Playlist..."), this);
+    newAction=new QAction(MonoIcon::icon(FontAwesome::asterisk, Utils::monoIconColor()), tr("New Playlist..."), this);
     connect(newAction, SIGNAL(triggered()), this, SIGNAL(addToNew()));
     Action::initIcon(newAction);
     alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=alignments[COL_PERFORMER]=int(Qt::AlignVCenter|Qt::AlignLeft);
     alignments[COL_LENGTH]=alignments[COL_YEAR]=int(Qt::AlignVCenter|Qt::AlignRight);
-    #endif
-    #if defined ENABLE_MODEL_TEST
-    new ModelTest(this, this);
-    #endif
 }
 
 PlaylistsModel::~PlaylistsModel()
 {
-    #ifndef ENABLE_UBUNTU
     if (itemMenu) {
         itemMenu->deleteLater();
         itemMenu=0;
     }
-    #endif
 }
 
 QString PlaylistsModel::name() const
@@ -125,12 +109,12 @@ QString PlaylistsModel::name() const
 
 QString PlaylistsModel::title() const
 {
-    return i18n("Stored Playlists");
+    return tr("Stored Playlists");
 }
 
 QString PlaylistsModel::descr() const
 {
-    return i18n("Standard playlists");
+    return tr("Standard playlists");
 }
 
 int PlaylistsModel::rowCount(const QModelIndex &index) const
@@ -216,7 +200,6 @@ QModelIndex PlaylistsModel::index(int row, int col, const QModelIndex &parent) c
     return row<items.count() ? createIndex(row, col, items.at(row)) : QModelIndex();
 }
 
-#ifndef ENABLE_UBUNTU
 QVariant PlaylistsModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
     if (Qt::Horizontal==orientation) {
@@ -259,7 +242,6 @@ bool PlaylistsModel::setHeaderData(int section, Qt::Orientation orientation, con
     }
     return false;
 }
-#endif
 
 QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
 {
@@ -275,11 +257,9 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
         return QVariant();
     }
 
-    #ifndef ENABLE_UBUNTU
     if (Qt::TextAlignmentRole==role) {
         return alignments[index.column()];
     }
-    #endif
 
     Item *item=static_cast<Item *>(index.internalPointer());
 
@@ -287,10 +267,6 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
         PlaylistItem *pl=static_cast<PlaylistItem *>(item);
 
         switch(role) {
-        #ifdef ENABLE_UBUNTU
-        case Cantata::Role_Image:
-            return QString();
-        #endif
         case Cantata::Role_ListImage:
             return false;
         case Cantata::Role_IsCollection:
@@ -357,20 +333,18 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             }
             return 0==pl->songs.count()
                 ? pl->visibleName()
-                : pl->visibleName()+"\n"+Plurals::tracksWithDuration(pl->songs.count(), Utils::formatTime(pl->totalTime()));
-        #ifndef ENABLE_UBUNTU
+                : pl->visibleName()+"\n"+tr("%n Tracks (%1)", "", pl->songs.count()).arg(Utils::formatTime(pl->totalTime()));
         case Qt::DecorationRole:
-            return multiCol ? QVariant() : (pl->isSmartPlaylist ? Icons::self()->dynamicRuleIcon : Icons::self()->playlistListIcon);
-        #endif
+            return multiCol ? QVariant() : (pl->isSmartPlaylist ? Icons::self()->dynamicListIcon : Icons::self()->playlistListIcon);
         case Cantata::Role_SubText:
             if (!pl->loaded) {
                 pl->loaded=true;
                 emit playlistInfo(pl->name);
             }
             if (pl->isSmartPlaylist) {
-                return i18n("Smart Playlist");
+                return tr("Smart Playlist");
             }
-            return Plurals::tracksWithDuration(pl->songs.count(), Utils::formatTime(pl->totalTime()));
+            return tr("%n Tracks (%1)", "", pl->songs.count()).arg(Utils::formatTime(pl->totalTime()));
         case Cantata::Role_TitleActions:
             return true;
         default:
@@ -380,10 +354,6 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
         SongItem *s=static_cast<SongItem *>(item);
 
         switch (role) {
-        #ifdef ENABLE_UBUNTU
-        case Cantata::Role_Image:
-            return QString();
-        #endif
         case Cantata::Role_ListImage:
             return true;
         case Cantata::Role_IsCollection:
@@ -480,14 +450,14 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
                 return QVariant();
             }
             return s->toolTip();
-        #ifndef ENABLE_UBUNTU
         case Qt::DecorationRole:
             return multiCol ? QVariant() : (s->title.isEmpty() ? Icons::self()->streamIcon : Icons::self()->audioListIcon);
-        #endif
         case Cantata::Role_MainText:
-            return s->title.isEmpty() ? s->file : s->title;
+            return s->title.isEmpty() ? s->name().isEmpty() ? s->file : s->name() : s->title;
         case Cantata::Role_SubText:
-            return s->artist+QLatin1String(" - ")+s->displayAlbum()+QLatin1String(" - ")+Utils::formatTime(s->time);
+            return (s->artist.isEmpty() ? QString() : (s->artist+QLatin1String(" - ")))+
+                   (s->displayAlbum().isEmpty() ? QString() : (s->displayAlbum()+QLatin1String(" - ")))+
+                   (s->time>0 ? Utils::formatTime(s->time) : QString());
         default:
             return ActionModel::data(index, role);
         }
@@ -496,7 +466,6 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
-#ifndef ENABLE_UBUNTU
 bool PlaylistsModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     if (Cantata::Role_DropAdjust==role) {
@@ -506,7 +475,6 @@ bool PlaylistsModel::setData(const QModelIndex &index, const QVariant &value, in
         return QAbstractItemModel::setData(index, value, role);
     }
 }
-#endif
 
 Qt::ItemFlags PlaylistsModel::flags(const QModelIndex &index) const
 {
@@ -521,6 +489,14 @@ Qt::DropActions PlaylistsModel::supportedDropActions() const
     return Qt::CopyAction | Qt::MoveAction;
 }
 
+static QString songFileName(const Song &song)
+{
+    if (song.isStream() && song.title.isEmpty()) {
+        return MPDParseUtils::addStreamName(song.file, song.name());
+    }
+    return song.file;
+}
+
 QStringList PlaylistsModel::filenames(const QModelIndexList &indexes, bool filesOnly) const
 {
     QStringList fnames;
@@ -537,7 +513,7 @@ QStringList PlaylistsModel::filenames(const QModelIndexList &indexes, bool files
             }
         } else if (!selectedPlaylists.contains(static_cast<SongItem*>(item)->parent)) {
             if (!filesOnly || !static_cast<SongItem*>(item)->file.contains(':')) {
-                fnames << static_cast<SongItem*>(item)->file;
+                fnames << songFileName(* static_cast<SongItem*>(item));
             }
         }
     }
@@ -572,7 +548,6 @@ QList<Song> PlaylistsModel::songs(const QModelIndexList &indexes) const
     return songs;
 }
 
-#ifndef ENABLE_UBUNTU
 static const QLatin1String constPlaylistNameMimeType("cantata/playlistnames");
 static const QLatin1String constPositionsMimeType("cantata/positions");
 static const char *constPlaylistProp="hasPlaylist";
@@ -702,7 +677,6 @@ QStringList PlaylistsModel::mimeTypes() const
     types << PlayQueueModel::constFileNameMimeType;
     return types;
 }
-#endif
 
 void PlaylistsModel::getPlaylists()
 {
@@ -717,7 +691,6 @@ void PlaylistsModel::clear()
     endResetModel();
 }
 
-#ifndef ENABLE_UBUNTU
 MirrorMenu * PlaylistsModel::menu()
 {
     if (!itemMenu) {
@@ -725,7 +698,6 @@ MirrorMenu * PlaylistsModel::menu()
     }
     return itemMenu;
 }
-#endif
 
 void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
 {
@@ -739,14 +711,8 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         }
         endResetModel();
         updateItemMenu();
-        #ifdef ENABLE_UBUNTU
-        emit updated();
-        #endif
     } else if (playlists.isEmpty()) {
         clear();
-        #ifdef ENABLE_UBUNTU
-        emit updated();
-        #endif
     } else {
         QModelIndex parent=QModelIndex();
         QSet<QString> existing;
@@ -799,9 +765,6 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
 
         if (added.count() || removed.count()) {
             updateItemMenu();
-            #ifdef ENABLE_UBUNTU
-            emit updated();
-            #endif
         }
     }
 }
@@ -812,51 +775,25 @@ void PlaylistsModel::playlistInfoRetrieved(const QString &name, const QList<Song
 
     if (pl) {
         QModelIndex idx=createIndex(items.indexOf(pl), 0, pl);
-        if (pl->songs.isEmpty()) {
-            if (songs.isEmpty()) {
-                return;
-            }
+
+        // If Cantata moves songs, then movedInPlaylist() is called. Likwise if
+        // Cantata deltes songs, removedFromPlaylist() is called. Therefore, no
+        // need to do partial updates here. If another client does a move whilst
+        // a playlist has duplicate songs, then it can fail. #873
+        if (!pl->songs.isEmpty()) {
+            beginRemoveRows(idx, 0, pl->songs.count()-1);
+            pl->clearSongs();
+            endRemoveRows();
+        }
+
+        if (!songs.isEmpty()) {
             beginInsertRows(idx, 0, songs.count()-1);
             foreach (const Song &s, songs) {
                 pl->songs.append(new SongItem(s, pl));
             }
             endInsertRows();
-        } else if (songs.isEmpty()) {
-            beginRemoveRows(idx, 0, pl->songs.count()-1);
-            pl->clearSongs();
-            endRemoveRows();
-        } else {
-            for (qint32 i=0; i<songs.count(); ++i) {
-                Song s=songs.at(i);
-                SongItem *si=i<pl->songs.count() ? pl->songs.at(i) : 0;
-                if (i>=pl->songs.count() || !(s==*static_cast<Song *>(si))) {
-                    si=i<pl->songs.count() ? pl->getSong(s, i) : 0;
-                    if (!si) {
-                        beginInsertRows(idx, i, i);
-                        pl->songs.insert(i, new SongItem(s, pl));
-                        endInsertRows();
-                    } else {
-                        int existing=pl->songs.indexOf(si);
-                        beginMoveRows(idx, existing, existing, idx, i>existing ? i+1 : i);
-                        SongItem *si=pl->songs.takeAt(existing);
-                        si->key=s.key;
-                        pl->songs.insert(i, si);
-                        endMoveRows();
-                    }
-                } else if (si) {
-                    si->key=s.key;
-                }
-            }
-
-            if (pl->songs.count()>songs.count()) {
-                int toRemove=pl->songs.count()-songs.count();
-                beginRemoveRows(idx, pl->songs.count()-toRemove, pl->songs.count()-1);
-                for (int i=0; i<toRemove; ++i) {
-                    delete pl->songs.takeLast();
-                }
-                endRemoveRows();
-            }
         }
+
         emit updated(idx);
         emit dataChanged(idx, idx);
     } else {
@@ -960,9 +897,6 @@ void PlaylistsModel::mpdConnectionStateChanged(bool connected)
 void PlaylistsModel::coverLoaded(const Song &song, int s)
 {
     Q_UNUSED(s)
-    #ifdef ENABLE_UBUNTU
-    Q_UNUSED(song)
-    #else
     if (!song.isArtistImageRequest() && !song.isComposerImageRequest()) {
         int plRow=0;
         foreach (const PlaylistItem *pl, items) {
@@ -981,12 +915,10 @@ void PlaylistsModel::coverLoaded(const Song &song, int s)
             plRow++;
         }
     }
-    #endif
 }
 
 void PlaylistsModel::updateItemMenu(bool create)
 {
-    #ifndef ENABLE_UBUNTU
     if (!itemMenu) {
         if (!create) {
             return;
@@ -1006,7 +938,6 @@ void PlaylistsModel::updateItemMenu(bool create)
     foreach (const QString &n, names) {
         itemMenu->addAction(n, this, SLOT(emitAddToExisting()));
     }
-    #endif
 }
 
 PlaylistsModel::PlaylistItem * PlaylistsModel::getPlaylist(const QString &name)
@@ -1049,7 +980,7 @@ PlaylistsModel::PlaylistItem::PlaylistItem(const Playlist &pl, quint32 k)
     , key(k)
     , lastModified(pl.lastModified)
 {
-    loaded=isSmartPlaylist=MPDConnection::self()->isMopdidy() && name.startsWith("Smart Playlist:");
+    loaded=isSmartPlaylist=MPDConnection::self()->isMopidy() && name.startsWith("Smart Playlist:");
     if (isSmartPlaylist) {
         shortName=name.mid(16);
     }
