@@ -1,8 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
- * Copyright (c) 2014 Niklas Wenzel <nikwen.developer@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -25,7 +24,6 @@
  * along with QtMPC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "support/localize.h"
 #include "playqueuemodel.h"
 #include "mpd-interface/mpdconnection.h"
 #include "mpd-interface/mpdparseutils.h"
@@ -36,6 +34,7 @@
 #include "http/httpserver.h"
 #include "gui/settings.h"
 #include "support/icon.h"
+#include "support/monoicon.h"
 #include "support/utils.h"
 #include "config.h"
 #include "support/action.h"
@@ -43,6 +42,7 @@
 #include "support/globalstatic.h"
 #include "gui/covers.h"
 #include "widgets/groupedview.h"
+#include "widgets/icons.h"
 #include "roles.h"
 #ifdef ENABLE_DEVICES_SUPPORT
 #include "devicesmodel.h"
@@ -61,13 +61,8 @@
 #include <QApplication>
 #include <QMenu>
 
-#if defined ENABLE_MODEL_TEST
-#include "modeltest.h"
-#endif
-
 GLOBAL_STATIC(PlayQueueModel, instance)
 
-#ifndef ENABLE_UBUNTU
 const QLatin1String PlayQueueModel::constMoveMimeType("cantata/move");
 const QLatin1String PlayQueueModel::constFileNameMimeType("cantata/filename");
 const QLatin1String PlayQueueModel::constUriMimeType("text/uri-list");
@@ -80,6 +75,8 @@ static const QLatin1String constSortByGenreKey("genre");
 static const QLatin1String constSortByYearKey("year");
 static const QLatin1String constSortByComposerKey("composer");
 static const QLatin1String constSortByPerformerKey("performer");
+static const QLatin1String constSortByTitleKey("title");
+static const QLatin1String constSortByNumberKey("track");
 
 static bool checkExtension(const QString &file)
 {
@@ -108,7 +105,7 @@ static QStringList parseUrls(const QStringList &urls, bool percentEncoded)
         if (QLatin1String("http")==u.scheme()) {
             useable.append(u.toString());
         } else if ((u.scheme().isEmpty() || QLatin1String("file")==u.scheme()) && checkExtension(u.path())) {
-            if (!HttpServer::self()->forceUsage() && MPDConnection::self()->localFilePlaybackSupported() && !u.path().startsWith(QLatin1String("/media/"))) {
+            if (MPDConnection::self()->localFilePlaybackSupported() && !u.path().startsWith(QLatin1String("/media/"))) {
                 useable.append(QLatin1String("file://")+u.path());
             } else if (HttpServer::self()->isAlive()) {
                 useable.append(HttpServer::self()->encodeUrl(u.path()));
@@ -169,22 +166,22 @@ QList<quint32> PlayQueueModel::decodeInts(const QMimeData &mimeData, const QStri
 QString PlayQueueModel::headerText(int col)
 {
     switch (col) {
-    case COL_TITLE:     return i18n("Title");
-    case COL_ARTIST:    return i18n("Artist");
-    case COL_ALBUM:     return i18n("Album");
-    case COL_TRACK:     return i18nc("Track number", "#");
-    case COL_LENGTH:    return i18n("Length");
-    case COL_DISC:      return i18n("Disc");
-    case COL_YEAR:      return i18n("Year");
-    case COL_GENRE:     return i18n("Genre");
-    case COL_PRIO:      return i18n("Priority");
-    case COL_COMPOSER:  return i18n("Composer");
-    case COL_PERFORMER: return i18n("Performer");
-    case COL_RATING:    return i18n("Rating");
+    case COL_TITLE:     return tr("Title");
+    case COL_ARTIST:    return tr("Artist");
+    case COL_ALBUM:     return tr("Album");
+    case COL_TRACK:     return tr("#", "Track number");
+    case COL_LENGTH:    return tr("Length");
+    case COL_DISC:      return tr("Disc");
+    case COL_YEAR:      return tr("Year");
+    case COL_ORIGYEAR:  return tr("Original Year");
+    case COL_GENRE:     return tr("Genre");
+    case COL_PRIO:      return tr("Priority");
+    case COL_COMPOSER:  return tr("Composer");
+    case COL_PERFORMER: return tr("Performer");
+    case COL_RATING:    return tr("Rating");
     default:            return QString();
     }
 }
-#endif
 
 PlayQueueModel::PlayQueueModel(QObject *parent)
     : QAbstractItemModel(parent)
@@ -194,16 +191,10 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     , mpdState(MPDState_Inactive)
     , stopAfterCurrent(false)
     , stopAfterTrackId(-1)
-    #ifdef ENABLE_UBUNTU
-    , undoLimit(0)
-    #else
-    , undoLimit(Settings::self()->undoSteps())
-    #endif
+    , undoLimit(10)
     , undoEnabled(undoLimit>0)
     , lastCommand(Cmd_Other)
-    #ifndef ENABLE_UBUNTU
     , dropAdjust(0)
-    #endif
 {
     fetcher=new StreamFetcher(this);
     connect(this, SIGNAL(modelReset()), this, SLOT(stats()));
@@ -231,51 +222,45 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     #ifdef ENABLE_DEVICES_SUPPORT //TODO: Problems here with devices support!!!
     connect(DevicesModel::self(), SIGNAL(updatedDetails(QList<Song>)), SLOT(updateDetails(QList<Song>)));
     #endif
-    #if defined ENABLE_MODEL_TEST
-    new ModelTest(this, this);
-    #endif
-
-    #ifdef ENABLE_UBUNTU
-    connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)), this, 
-            SLOT(setCover(const Song &, const QImage &, const QString &)));
-    #else
 
     UNITY_MENU_ICON_CHECK
-    removeDuplicatesAction=new Action(i18n("Remove Duplicates"), this);
+    removeDuplicatesAction=new Action(tr("Remove Duplicates"), this);
     removeDuplicatesAction->setEnabled(false);
-    undoAction=ActionCollection::get()->createAction("playqueue-undo", i18n("Undo"), HIDE_MENU_ICON_NAME("edit-undo"));
+    QColor col=Utils::monoIconColor();
+    undoAction=ActionCollection::get()->createAction("playqueue-undo", tr("Undo"), HIDE_MENU_ICON(MonoIcon::icon(FontAwesome::undo, col)));
     undoAction->setShortcut(Qt::ControlModifier+Qt::Key_Z);
-    redoAction=ActionCollection::get()->createAction("playqueue-redo", i18n("Redo"), HIDE_MENU_ICON_NAME("edit-redo"));
+    redoAction=ActionCollection::get()->createAction("playqueue-redo", tr("Redo"), HIDE_MENU_ICON(MonoIcon::icon(FontAwesome::repeat, col)));
     redoAction->setShortcut(Qt::ControlModifier+Qt::ShiftModifier+Qt::Key_Z);
     connect(undoAction, SIGNAL(triggered()), this, SLOT(undo()));
     connect(redoAction, SIGNAL(triggered()), this, SLOT(redo()));
     connect(removeDuplicatesAction, SIGNAL(triggered()), this, SLOT(removeDuplicates()));
 
-    shuffleAction=new Action(i18n("Shuffle"), this);
+    shuffleAction=new Action(tr("Shuffle"), this);
     shuffleAction->setMenu(new QMenu(0));
-    Action *shuffleTracksAction = new Action(i18n("Tracks"), shuffleAction);
-    Action *shuffleAlbumsAction = new Action(i18n("Albums"), shuffleAction);
+    Action *shuffleTracksAction = new Action(tr("Tracks"), shuffleAction);
+    Action *shuffleAlbumsAction = new Action(tr("Albums"), shuffleAction);
     connect(shuffleTracksAction, SIGNAL(triggered()), MPDConnection::self(), SLOT(shuffle()));
     connect(shuffleAlbumsAction, SIGNAL(triggered()), this, SLOT(shuffleAlbums()));
     shuffleAction->menu()->addAction(shuffleTracksAction);
     shuffleAction->menu()->addAction(shuffleAlbumsAction);
 
-    sortAction=new Action(i18n("Sort By"), this);
+    sortAction=new Action(tr("Sort By"), this);
     sortAction->setMenu(new QMenu(0));
-    addSortAction(i18n("Artist"), constSortByArtistKey);
-    addSortAction(i18n("Album Artist"), constSortByAlbumArtistKey);
-    addSortAction(i18n("Album"), constSortByAlbumKey);
-    addSortAction(i18n("Genre"), constSortByGenreKey);
-    addSortAction(i18n("Year"), constSortByYearKey);
-    addSortAction(i18n("Composer"), constSortByComposerKey);
-    addSortAction(i18n("Performer"), constSortByPerformerKey);
+    addSortAction(tr("Artist"), constSortByArtistKey);
+    addSortAction(tr("Album Artist"), constSortByAlbumArtistKey);
+    addSortAction(tr("Album"), constSortByAlbumKey);
+    addSortAction(tr("Track Title"), constSortByTitleKey);
+    addSortAction(tr("Track Number"), constSortByNumberKey);
+    addSortAction(tr("Genre"), constSortByGenreKey);
+    addSortAction(tr("Year"), constSortByYearKey);
+    addSortAction(tr("Composer"), constSortByComposerKey);
+    addSortAction(tr("Performer"), constSortByPerformerKey);
     controlActions();
     shuffleAction->setEnabled(false);
     sortAction->setEnabled(false);
     alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=alignments[COL_PERFORMER]=int(Qt::AlignVCenter|Qt::AlignLeft);
-    alignments[COL_TRACK]=alignments[COL_LENGTH]=alignments[COL_DISC]=alignments[COL_YEAR]=alignments[COL_PRIO]=int(Qt::AlignVCenter|Qt::AlignRight);
+    alignments[COL_TRACK]=alignments[COL_LENGTH]=alignments[COL_DISC]=alignments[COL_YEAR]=alignments[COL_ORIGYEAR]=alignments[COL_PRIO]=int(Qt::AlignVCenter|Qt::AlignRight);
     alignments[COL_RATING]=int(Qt::AlignVCenter|Qt::AlignHCenter);
-    #endif
 }
 
 PlayQueueModel::~PlayQueueModel()
@@ -293,7 +278,6 @@ QModelIndex PlayQueueModel::parent(const QModelIndex &idx) const
     return QModelIndex();
 }
 
-#ifndef ENABLE_UBUNTU
 QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
     if (Qt::Horizontal==orientation) {
@@ -303,7 +287,7 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
         case Qt::TextAlignmentRole:
             return alignments[section];
         case Cantata::Role_InitiallyHidden:
-            return COL_YEAR==section || COL_DISC==section || COL_GENRE==section || COL_PRIO==section ||
+            return COL_YEAR==section || COL_ORIGYEAR==section || COL_DISC==section || COL_GENRE==section || COL_PRIO==section ||
                    COL_COMPOSER==section || COL_PERFORMER==section || COL_RATING==section;
         case Cantata::Role_Hideable:
             return COL_TITLE!=section && COL_ARTIST!=section;
@@ -316,6 +300,7 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
             case COL_ALBUM:     return 0.27;
             case COL_LENGTH:    return 0.05;
             case COL_YEAR:      return 0.05;
+            case COL_ORIGYEAR:  return 0.05;
             case COL_GENRE:     return 0.1;
             case COL_PRIO:      return 0.015;
             case COL_COMPOSER:  return 0.2;
@@ -323,7 +308,7 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
             case COL_RATING:    return 0.08;
             }
         case Cantata::Role_ContextMenuText:
-            return COL_TRACK==section ? i18n("# (Track Number)") : headerText(section);
+            return COL_TRACK==section ? tr("# (Track Number)") : headerText(section);
         default:
             break;
         }
@@ -343,7 +328,6 @@ bool PlayQueueModel::setHeaderData(int section, Qt::Orientation orientation, con
     }
     return false;
 }
-#endif
 
 int PlayQueueModel::rowCount(const QModelIndex &idx) const
 {
@@ -365,21 +349,6 @@ static QString basicPath(const Song &song)
     return -1==marker ? path : path.left(marker);
 }
 
-#ifdef ENABLE_UBUNTU
-static const QString constDefaultCover=QLatin1String("qrc:/album.svg");
-
-//Expose role names, so that they can be accessed via QML
-QHash<int, QByteArray> PlayQueueModel::roleNames() const
-{
-    QHash<int, QByteArray> roles;
-    roles[Cantata::Role_MainText] = "mainText";
-    roles[Cantata::Role_SubText] = "subText";
-    roles[Cantata::Role_Image] = "image";
-    roles[Cantata::Role_Time] = "time";
-    return roles;
-}
-#endif
-
 QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid() && Cantata::Role_RatingCol==role) {
@@ -403,22 +372,6 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         const Song &s=songs.at(index.row());
         return s.time>0 ? Utils::formatTime(s.time) : QLatin1String("");
     }
-    #ifdef ENABLE_UBUNTU
-    case Cantata::Role_Image: {
-        Song s=songs.at(index.row());
-        QMap<quint16, QString>::ConstIterator it=covers.find(s.key);
-        if (it!=covers.constEnd()) {
-            return it.value().isEmpty() ? constDefaultCover : it.value();
-        }
-        QString fileName=Covers::self()->requestImage(s).fileName;
-        covers.insert(s.key, fileName);
-        if (fileName.isEmpty()) {
-            coverRequests.insert(s.key, s);
-            return constDefaultCover;
-        }
-        return fileName;
-    }
-    #endif
     case Cantata::Role_IsCollection:
         return false;
     case Cantata::Role_CollectionId:
@@ -572,6 +525,11 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
                 return QVariant();
             }
             return song.year;
+        case COL_ORIGYEAR:
+            if (song.origYear <= 0) {
+                return QVariant();
+            }
+            return song.origYear;
         case COL_GENRE:
             return song.displayGenre();
         case COL_PRIO:
@@ -606,7 +564,6 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
             return s.toolTip();
         }
     }
-    #ifndef ENABLE_UBUNTU
     case Qt::TextAlignmentRole:
         return alignments[index.column()];
     case Cantata::Role_Decoration: {
@@ -614,16 +571,15 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         if (id==currentSongId) {
             switch (mpdState) {
             case MPDState_Inactive:
-            case MPDState_Stopped: return Icon("media-playback-stop");
-            case MPDState_Playing: return Icon(stopAfterCurrent ? "media-playback-stop" : "media-playback-start");
-            case MPDState_Paused:  return Icon("media-playback-pause");
+            case MPDState_Stopped: return MonoIcon::icon(FontAwesome::stop, Utils::monoIconColor());
+            case MPDState_Playing: return MonoIcon::icon(stopAfterCurrent ? FontAwesome::playcircle : FontAwesome::play, Utils::monoIconColor());
+            case MPDState_Paused:  return MonoIcon::icon(FontAwesome::pause, Utils::monoIconColor());
             }
         } else if (-1!=id && id==stopAfterTrackId) {
-            return Icon("media-playback-stop");
+            return MonoIcon::icon(FontAwesome::stop, Utils::monoIconColor());
         }
         break;
     }
-    #endif
     default:
         break;
     }
@@ -631,7 +587,6 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
-#ifndef ENABLE_UBUNTU
 bool PlayQueueModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     if (Cantata::Role_DropAdjust==role) {
@@ -641,7 +596,6 @@ bool PlayQueueModel::setData(const QModelIndex &index, const QVariant &value, in
         return QAbstractItemModel::setData(index, value, role);
     }
 }
-#endif
 
 Qt::DropActions PlayQueueModel::supportedDropActions() const
 {
@@ -656,7 +610,6 @@ Qt::ItemFlags PlayQueueModel::flags(const QModelIndex &index) const
     return Qt::ItemIsDropEnabled;
 }
 
-#ifndef ENABLE_UBUNTU
 /**
  * @return A QStringList with the mimetypes we support
  */
@@ -728,18 +681,13 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
     }
     return false;
 }
-#endif
 
 void PlayQueueModel::load(const QStringList &urls)
 {
-    #ifdef ENABLE_UBUNTU
-    Q_UNUSED(urls)
-    #else
     QStringList useable=parseUrls(urls, false);
     if (useable.count()) {
         addItems(useable, songs.count(), songs.isEmpty(), 0);
     }
-    #endif
 }
 
 void PlayQueueModel::addItems(const QStringList &items, int row, int action, quint8 priority)
@@ -861,7 +809,7 @@ void PlayQueueModel::updateCurrentSong(quint32 id)
     }
 
     currentSongRowNum=getRowById(currentSongId);
-    if (currentSongRowNum>0 && currentSongRowNum<=songs.count()) {
+    if (currentSongRowNum>=0 && currentSongRowNum<=songs.count()) {
         const Song &song=songs.at(currentSongRowNum);
         if (Song::Rating_Null==song.rating) {
             song.rating=Song::Rating_Requested;
@@ -917,24 +865,15 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
     currentSongRowNum=-1;
     if (songList.isEmpty()) {
         Song::clearKeyStore(MPDParseUtils::Loc_PlayQueue);
-        #ifdef ENABLE_UBUNTU
-        covers.clear();
-        coverRequests.clear();
-        #endif
     }
 
-    #ifndef ENABLE_UBUNTU
     removeDuplicatesAction->setEnabled(songList.count()>1);
-    #endif
     QList<Song> prev;
     if (undoEnabled) {
         prev=songs;
     }
 
     QSet<qint32> newIds;
-    #ifdef ENABLE_UBUNTU
-    QSet<quint16> currentKeys;
-    #endif
     foreach (const Song &s, songList) {
         newIds.insert(s.id);
     }
@@ -952,12 +891,6 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
         if (songList.isEmpty()) {
             stopAfterTrackId=-1;
         }
-        
-        #ifdef ENABLE_UBUNTU
-        foreach (const Song &s, songs) {
-            currentKeys.insert(s.key);
-        }
-        #endif
     } else {
         time = 0;
 
@@ -981,9 +914,6 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
                 if (-1==existingPos) {
                     beginInsertRows(QModelIndex(), i, i);
                     songs.insert(i, s);
-                    #ifdef ENABLE_UBUNTU
-                    currentKeys.insert(s.key);
-                    #endif
                     endInsertRows();
                 } else {
                     beginMoveRows(QModelIndex(), existingPos, existingPos, QModelIndex(), i>existingPos ? i+1 : i);
@@ -991,22 +921,13 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
 //                     old.pos=s.pos;
                     s.rating=old.rating;
                     songs.insert(i, isEmpty ? old : s);
-                    #ifdef ENABLE_UBUNTU
-                    currentKeys.insert(isEmpty ? old.key : s.key);
-                    #endif
                     endMoveRows();
                 }
             } else if (isEmpty) {
                 s=currentSongAtPos;
-                #ifdef ENABLE_UBUNTU
-                currentKeys.insert(s.key);
-                #endif
             } else {
                 s.key=currentSongAtPos.key;
                 s.rating=currentSongAtPos.rating;
-                #ifdef ENABLE_UBUNTU
-                currentKeys.insert(s.key);
-                #endif
                 songs.replace(i, s);
                 if (s.title!=currentSongAtPos.title || s.artist!=currentSongAtPos.artist || s.name()!=currentSongAtPos.name()) {
                     emit dataChanged(index(i, 0), index(i, columnCount(QModelIndex())-1));
@@ -1036,18 +957,8 @@ void PlayQueueModel::update(const QList<Song> &songList, bool isComplete)
     }
 
     saveHistory(prev);
-    #ifdef ENABLE_UBUNTU
-    QSet<quint16> removedKeys=covers.keys().toSet()-currentKeys;
-    if (!removedKeys.isEmpty()) {
-        foreach (quint16 k, removedKeys) {
-            covers.remove(k);
-            coverRequests.remove(k);
-        }
-    }
-    #else
     shuffleAction->setEnabled(songs.count()>1);
     sortAction->setEnabled(songs.count()>1);
-    #endif
 }
 
 void PlayQueueModel::setStopAfterTrack(qint32 track)
@@ -1145,9 +1056,7 @@ void PlayQueueModel::enableUndo(bool e)
         undoStack.clear();
         redoStack.clear();
     }
-    #ifndef ENABLE_UBUNTU
     controlActions();
-    #endif
 }
 
 static PlayQueueModel::UndoItem getState(const QList<Song> &songs)
@@ -1226,13 +1135,10 @@ void PlayQueueModel::saveHistory(const QList<Song> &prevList)
         }
     }
 
-    #ifndef ENABLE_UBUNTU
     controlActions();
-    #endif
     lastCommand=Cmd_Other;
 }
 
-#ifndef ENABLE_UBUNTU
 void PlayQueueModel::controlActions()
 {
     undoAction->setEnabled(!undoStack.isEmpty());
@@ -1291,7 +1197,7 @@ static bool albumSort(const Song *s1, const Song *s2)
 
 static bool genreSort(const Song *s1, const Song *s2)
 {
-    int c=s1->genre.localeAwareCompare(s2->genre);
+    int c=s1->compareGenres(*s2);
     return c<0 || (c==0 && (*s1)<(*s2));
 }
 
@@ -1299,11 +1205,20 @@ static bool yearSort(const Song *s1, const Song *s2)
 {
     return s1->year<s2->year || (s1->year==s2->year && (*s1)<(*s2));
 }
-#endif
+
+static bool titleSort(const Song *s1, const Song *s2)
+{
+    int c=s1->title.localeAwareCompare(s2->title);
+    return c<0 || (c==0 && (*s1)<(*s2));
+}
+
+static bool trackSort(const Song *s1, const Song *s2)
+{
+    return s1->track<s2->track || (s1->track==s2->track && (*s1)<(*s2));
+}
 
 void PlayQueueModel::sortBy()
 {
-    #ifndef ENABLE_UBUNTU
     Action *act=qobject_cast<Action *>(sender());
     if (act) {
         QString key=act->property(constSortByKey).toString();
@@ -1327,6 +1242,10 @@ void PlayQueueModel::sortBy()
             qSort(copy.begin(), copy.end(), composerSort);
         } else if (constSortByPerformerKey==key) {
             qSort(copy.begin(), copy.end(), performerSort);
+        } else if (constSortByTitleKey==key) {
+            qSort(copy.begin(), copy.end(), titleSort);
+        } else if (constSortByNumberKey==key) {
+            qSort(copy.begin(), copy.end(), trackSort);
         }
         QList<quint32> positions;
         foreach (const Song *s, copy) {
@@ -1334,7 +1253,6 @@ void PlayQueueModel::sortBy()
         }
         emit setOrder(positions);
     }
-    #endif
 }
 
 void PlayQueueModel::removeDuplicates()
@@ -1388,60 +1306,6 @@ void PlayQueueModel::stickerDbChanged()
     }
 }
 
-void PlayQueueModel::setCover(const Song &song, const QImage &img, const QString &file)
-{
-    Q_UNUSED(img)
-    #ifdef ENABLE_UBUNTU
-    if (file.isEmpty() || coverRequests.isEmpty()) {
-        return;
-    }
-    
-    // In case of multi-disc albums we will have multiple 'keys' mapped to a single artist+album combo.
-    // Therefore, we need to find all keys that have this mapping.
-    //
-    // Also, library and albums models might have requested the cover at the same time as playqueue, in which
-    // case the incomming song would have no key set
-    QList<quint16> keys;
-    QMap<quint16, Song>::ConstIterator it=coverRequests.constBegin();
-    QMap<quint16, Song>::ConstIterator end=coverRequests.constEnd();
-    for (; it!=end; ++it) {
-        if (song.albumArtist()==it.value().albumArtist() && song.album==it.value().album) {
-            keys.append(it.key());
-        }
-    }
-
-    foreach (quint16 key, keys) {
-        coverRequests.remove(key);
-        QMap<quint16, QString>::ConstIterator it=covers.find(key);
-        if (it!=covers.end() && it.value().isEmpty()) {
-            coverRequests.remove(it.key());
-            covers[key]="file://"+file;
-            int start=-1;
-            int end=-1;
-            for (int i=0; i<songs.size(); ++i) {
-                if (songs.at(i).key==key) {
-                    if (-1==start) {
-                        start=end=i;
-                    } else {
-                        end++;
-                    }
-                } else if (-1!=end) {
-                    emit dataChanged(index(start, 0), index(end, columnCount(QModelIndex())-1));
-                    start=end=-1;
-                }
-            }
-            if (-1!=end) {
-                emit dataChanged(index(start, 0), index(end, columnCount(QModelIndex())-1));
-                start=end=-1;
-            }
-        }
-    }
-    #else
-    Q_UNUSED(song)
-    Q_UNUSED(file)
-    #endif
-}
-
 void PlayQueueModel::undo()
 {
     if (!undoEnabled || undoStack.isEmpty()) {
