@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -26,18 +26,13 @@
 
 #include "mpdconnection.h"
 #include "mpdparseutils.h"
-#ifndef CANTATA_WEB
 #include "models/streamsmodel.h"
-#endif
 #ifdef ENABLE_SIMPLE_MPD_SUPPORT
 #include "mpduser.h"
+#endif
 #include "gui/settings.h"
-#endif
-#include "support/localize.h"
 #include "support/globalstatic.h"
-#if !defined ENABLE_UBUNTU && !defined CANTATA_WEB
 #include "support/configuration.h"
-#endif
 #include <QStringList>
 #include <QTimer>
 #include <QDir>
@@ -46,6 +41,7 @@
 #include <QPropertyAnimation>
 #include <QCoreApplication>
 #include <QUdpSocket>
+#include <complex>
 #include "support/thread.h"
 #include "cuefile.h"
 #if defined Q_OS_LINUX && defined QT_QTDBUS_FOUND
@@ -66,8 +62,8 @@ void MPDConnection::enableDebug()
 // #define REPORT_MPD_ERRORS
 static const int constSocketCommsTimeout=2000;
 static const int constMaxReadAttempts=4;
-static int maxFilesPerAddCommand=10000;
-static int seekStep=5;
+static const int constMaxFilesPerAddCommand=2000;
+static const int constConnTimer=5000;
 
 static const QByteArray constOkValue("OK");
 static const QByteArray constOkMpdValue("OK MPD");
@@ -85,15 +81,13 @@ static const QByteArray constIdleOutputValue("output");
 static const QByteArray constIdleStickerValue("sticker");
 static const QByteArray constIdleSubscriptionValue("subscription");
 static const QByteArray constIdleMessageValue("message");
-#ifndef CANTATA_WEB
 static const QByteArray constDynamicIn("cantata-dynamic-in");
 static const QByteArray constDynamicOut("cantata-dynamic-out");
-#endif
 static const QByteArray constRatingSticker("rating");
 
 static inline int socketTimeout(int dataSize)
 {
-    static const int constDataBlock=100000;
+    static const int constDataBlock=256;
     return ((dataSize/constDataBlock)+((dataSize%constDataBlock) ? 1 : 0))*constSocketCommsTimeout;
 }
 
@@ -124,14 +118,14 @@ QByteArray MPDConnection::encodeName(const QString &name)
     return '\"'+name.toUtf8().replace("\\", "\\\\").replace("\"", "\\\"")+'\"';
 }
 
-static QByteArray readFromSocket(MpdSocket &socket)
+static QByteArray readFromSocket(MpdSocket &socket, int timeout=constSocketCommsTimeout)
 {
     QByteArray data;
+    int attempt=0;
     while (QAbstractSocket::ConnectedState==socket.state()) {
-        int attempt=0;
         while (0==socket.bytesAvailable() && QAbstractSocket::ConnectedState==socket.state()) {
             DBUG << (void *)(&socket) << "Waiting for read data, attempt" << attempt;
-            if (socket.waitForReadyRead()) {
+            if (socket.waitForReadyRead(timeout)) {
                 break;
             }
             DBUG << (void *)(&socket) << "Wait for read failed - " << socket.errorString();
@@ -153,9 +147,9 @@ static QByteArray readFromSocket(MpdSocket &socket)
     return data;
 }
 
-static MPDConnection::Response readReply(MpdSocket &socket)
+static MPDConnection::Response readReply(MpdSocket &socket, int timeout=constSocketCommsTimeout)
 {
-    QByteArray data = readFromSocket(socket);
+    QByteArray data = readFromSocket(socket, timeout);
     return MPDConnection::Response(data.endsWith(constOkNlValue), data);
 }
 
@@ -174,7 +168,7 @@ QString MPDConnection::Response::getError(const QByteArray &command)
     if (!ok && data.size()>0) {
         int cmdEnd=data.indexOf("} ");
         if (-1==cmdEnd) {
-            return i18n("Unknown")+QLatin1String(" (")+command+QLatin1Char(')');
+            return tr("Unknown")+QLatin1String(" (")+command+QLatin1Char(')');
         } else {
             cmdEnd+=2;
             QString rv=data.mid(cmdEnd, data.length()-(data.endsWith('\n') ? cmdEnd+1 : cmdEnd));
@@ -207,18 +201,20 @@ MPDConnectionDetails::MPDConnectionDetails()
 QString MPDConnectionDetails::getName() const
 {
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
-    return name.isEmpty() ? i18n("Default") : (name==MPDUser::constName ? MPDUser::translatedName() : name);
+    return name.isEmpty() ? QObject::tr("Default") : (name==MPDUser::constName ? MPDUser::translatedName() : name);
     #else
-    return name.isEmpty() ? i18n("Default") : name;
+    return name.isEmpty() ? QObject::tr("Default") : name;
     #endif
 }
 
 QString MPDConnectionDetails::description() const
 {
-    if (hostname.startsWith('/')) {
-        return i18nc("name (host)", "\"%1\"", getName());
+    if (hostname.isEmpty()) {
+        return getName();
+    } else if (hostname.startsWith('/') || hostname.startsWith('~')) {
+        return getName();
     } else {
-        return i18nc("name (host:port)", "\"%1\" (%2:%3)", getName(), hostname, QString::number(port)); // Use QString::number to prevent KDE's i18n converting 6600 to 6,600!
+        return QObject::tr("\"%1\" (%2:%3)", "name (host:port)").arg(getName()).arg(hostname).arg(QString::number(port));
     }
 }
 
@@ -270,12 +266,7 @@ MPDConnection::MPDConnection()
     #if (defined Q_OS_LINUX && defined QT_QTDBUS_FOUND) || (defined Q_OS_MAC && defined IOKIT_FOUND)
     connect(PowerManagement::self(), SIGNAL(resuming()), this, SLOT(reconnect()));
     #endif
-    #if !defined ENABLE_UBUNTU && !defined CANTATA_WEB
-    Configuration cfg;
-    maxFilesPerAddCommand=cfg.get("mpdListSize", 10000, 100, 65535);
-    seekStep=cfg.get("seekStep", 5, 2, 60);
-    MPDParseUtils::setSingleTracksFolders(cfg.get("singleTracksFolders", QStringList()).toSet());
-    #endif
+    MPDParseUtils::setSingleTracksFolders(Configuration().get("singleTracksFolders", QStringList()).toSet());
 }
 
 MPDConnection::~MPDConnection()
@@ -299,12 +290,14 @@ void MPDConnection::start()
         connTimer->setSingleShot(false);
         moveToThread(thread);
         connect(thread, SIGNAL(finished()), connTimer, SLOT(stop()));
+        connect(connTimer, SIGNAL(timeout()), SLOT(getStatus()));
         thread->start();
     }
 }
 
 void MPDConnection::stop()
 {
+    stopPlaying();
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (details.name==MPDUser::constName && Settings::self()->stopOnExit()) {
         MPDUser::self()->stop();
@@ -374,10 +367,8 @@ MPDConnection::ConnectionReturn MPDConnection::connectToMPD(MpdSocket &socket, b
             }
 
             if (enableIdle) {
-                #ifndef CANTATA_WEB
                 dynamicId.clear();
                 setupRemoteDynamic();
-                #endif
                 connect(&socket, SIGNAL(readyRead()), this, SLOT(idleDataReady()), Qt::QueuedConnection);
                 connect(&socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SLOT(onSocketStateChanged(QAbstractSocket::SocketState)), Qt::QueuedConnection);
                 DBUG << (void *)(&socket) << "Enabling idle";
@@ -421,9 +412,9 @@ QString MPDConnection::errorString(ConnectionReturn status) const
     switch (status) {
     default:
     case Success:           return QString();
-    case Failed:            return i18n("Connection to %1 failed", details.description());
-    case ProxyError:        return i18n("Connection to %1 failed - please check your proxy settings", details.description());
-    case IncorrectPassword: return i18n("Connection to %1 failed - incorrect password", details.description());
+    case Failed:            return tr("Connection to %1 failed").arg(details.description());
+    case ProxyError:        return tr("Connection to %1 failed - please check your proxy settings").arg(details.description());
+    case IncorrectPassword: return tr("Connection to %1 failed - incorrect password").arg(details.description());
     }
 }
 
@@ -434,15 +425,29 @@ MPDConnection::ConnectionReturn MPDConnection::connectToMPD()
         DBUG << "Something has gone wrong with sockets, so disconnect";
         disconnectFromMPD();
     }
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
+    int maxConnAttempts=MPDUser::constName==details.name ? 2 : 1;
+    #else
+    int maxConnAttempts=1;
+    #endif
     ConnectionReturn status=Failed;
-    if (Success==(status=connectToMPD(sock)) && Success==(status=connectToMPD(idleSocket, true))) {
-        state=State_Connected;
-        emit socketAddress(sock.address());
-    } else {
-        disconnectFromMPD();
-        state=State_Disconnected;
+    for (int connAttempt=0; connAttempt<maxConnAttempts; ++connAttempt) {
+        if (Success==(status=connectToMPD(sock)) && Success==(status=connectToMPD(idleSocket, true))) {
+            state=State_Connected;
+            emit socketAddress(sock.address());
+        } else {
+            disconnectFromMPD();
+            state=State_Disconnected;
+            #ifdef ENABLE_SIMPLE_MPD_SUPPORT
+            if (0==connAttempt && MPDUser::constName==details.name) {
+                DBUG << "Restart personal mpd";
+                MPDUser::self()->stop();
+                MPDUser::self()->start();
+            }
+            #endif
+        }
     }
-    connTimer->start(30000);
+    connTimer->start(constConnTimer);
     return status;
 }
 
@@ -462,6 +467,14 @@ void MPDConnection::disconnectFromMPD()
     idleSocket.close();
     state=State_Disconnected;
     ver=0;
+    playQueueIds.clear();
+    streamIds.clear();
+    lastStatusPlayQueueVersion=0;
+    lastUpdatePlayQueueVersion=0;
+    currentSongId=0;
+    songPos=0;
+    mopidy=false;
+    isUpdatingDb=false;
     emit socketAddress(QString());
 }
 
@@ -499,7 +512,7 @@ void MPDConnection::reconnect()
         emit stateChanged(true);
         break;
     case Failed:
-        if (0==reconnectStart || abs(now-reconnectStart)<15) {
+        if (0==reconnectStart || std::abs(now-reconnectStart)<15) {
             if (0==reconnectStart) {
                 reconnectStart=now;
             }
@@ -508,8 +521,8 @@ void MPDConnection::reconnect()
                 reconnectTimer->setSingleShot(true);
                 connect(reconnectTimer, SIGNAL(timeout()), this, SLOT(reconnect()), Qt::QueuedConnection);
             }
-            if (abs(now-reconnectStart)>1) {
-                emit info(i18n("Connecting to %1", details.description()));
+            if (std::abs(now-reconnectStart)>1) {
+                emit info(tr("Connecting to %1").arg(details.description()));
             }
             reconnectTimer->start(500);
         } else {
@@ -536,7 +549,7 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
 
     #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     bool isUser=d.name==MPDUser::constName;
-    const MPDConnectionDetails &det=isUser ? MPDUser::self()->details() : d;
+    const MPDConnectionDetails &det=isUser ? MPDUser::self()->details(true) : d;
     #else
     const MPDConnectionDetails &det=d;
     #endif
@@ -548,6 +561,10 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
     #endif
 
     details=det;
+    // Issue #1041 - If this is a user MPD, then the call to MPDUser::self()->details() will clear the replayGain setting
+    // We can safely use that of the passed in details.
+    details.replayGain=d.replayGain;
+
     if (diffDetails || State_Connected!=state) {
         emit connectionChanged(details);
         DBUG << "setDetails" << details.hostname << details.port << (details.password.isEmpty() ? false : true);
@@ -572,6 +589,10 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
         ConnectionReturn status=connectToMPD();
         switch (status) {
         case Success:
+            // Issue #1041 - MPD does not seem to persist user/client made replaygain changes, so use the values read from Cantata's config.
+            if (replaygainSupported() && !details.replayGain.isEmpty()) {
+                sendCommand("replay_gain_mode "+details.replayGain.toLatin1());
+            }
             getStatus();
             getStats();
             getUrlHandlers();
@@ -581,12 +602,7 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
             emit stateChanged(true);
             break;
         default:
-            #ifdef CANTATA_WEB
-            qWarning() << "Could not connect to MPD" << details.hostname << details.port;
-            ::exit(-1);
-            #else
             emit error(errorString(status), true);
-            #endif
         }
     } else if (diffName) {
          emit stateChanged(true);
@@ -616,16 +632,15 @@ MPDConnection::Response MPDConnection::sendCommand(const QByteArray &command, bo
     DBUG << (void *)(&sock) << "sendCommand:" << log(command) << emitErrors << retry;
 
     if (!isConnected()) {
-        emit error(i18n("Failed to send command to %1 - not connected", details.description()), true);
+        emit error(tr("Failed to send command to %1 - not connected").arg(details.description()), true);
         return Response(false);
     }
 
     if (QAbstractSocket::ConnectedState!=sock.state() && !reconnected) {
         DBUG << (void *)(&sock) << "Socket (state:" << sock.state() << ") need to reconnect";
         sock.close();
-        ConnectionReturn status=connectToMPD(sock);
+        ConnectionReturn status=connectToMPD();
         if (Success!=status) {
-            // Failed to connect, so close *both* sockets!
             disconnectFromMPD();
             emit stateChanged(false);
             emit error(errorString(status), true);
@@ -650,7 +665,7 @@ MPDConnection::Response MPDConnection::sendCommand(const QByteArray &command, bo
         DBUG << "Timeout (ms):" << timeout;
         sock.waitForBytesWritten(timeout);
         DBUG << "Socket state after write:" << (int)sock.state();
-        response=readReply(sock);
+        response=readReply(sock, timeout);
     }
 
     if (!response.ok) {
@@ -666,40 +681,39 @@ MPDConnection::Response MPDConnection::sendCommand(const QByteArray &command, bo
             bool emitError=true;
             // Mopidy returns "incorrect arguments" for commands it does not support. The docs state that crossfade and replaygain mode
             // setting commands are not supported. So, if we get this error then just ignore it.
-            if ((command.startsWith("crossfade ") || command.startsWith("replay_gain_mode ")) &&
-                "incorrect arguments"==response.getError(command)) {
+            if (mopidy && (command.startsWith("crossfade ") || command.startsWith("replay_gain_mode "))) {
                 emitError=false;
             }
             if (emitError) {
                 if ((command.startsWith("add ") || command.startsWith("command_list_begin\nadd ")) && -1!=command.indexOf("\"file:///")) {
                     if (details.isLocal() && -1!=response.data.indexOf("Permission denied")) {
-                        emit error(i18n("Failed to load. Please check user \"mpd\" has read permission."));
+                        emit error(tr("Failed to load. Please check user \"mpd\" has read permission."));
                     } else if (!details.isLocal() && -1!=response.data.indexOf("Access denied")) {
-                        emit error(i18n("Failed to load. MPD can only play local files if connected via a local socket."));
+                        emit error(tr("Failed to load. MPD can only play local files if connected via a local socket."));
                     } else if (!response.getError(command).isEmpty()) {
-                        emit error(i18n("MPD reported the following error: %1", response.getError(command)));
+                        emit error(tr("MPD reported the following error: %1").arg(response.getError(command)));
                     } else {
                         disconnectFromMPD();
                         emit stateChanged(false);
-                        emit error(i18n("Failed to send command. Disconnected from %1", details.description()), true);
+                        emit error(tr("Failed to send command. Disconnected from %1").arg(details.description()), true);
                     }
                 } else if (!response.getError(command).isEmpty()) {
-                    emit error(i18n("MPD reported the following error: %1", response.getError(command)));
+                    emit error(tr("MPD reported the following error: %1").arg(response.getError(command)));
                 } /*else if ("listallinfo"==command && ver>=CANTATA_MAKE_VERSION(0,18,0)) {
                     disconnectFromMPD();
                     emit stateChanged(false);
-                    emit error(i18n("Failed to load library. Please increase \"max_output_buffer_size\" in MPD's config file."));
+                    emit error(tr("Failed to load library. Please increase \"max_output_buffer_size\" in MPD's config file."));
                 } */ else {
                     disconnectFromMPD();
                     emit stateChanged(false);
-                    emit error(i18n("Failed to send command. Disconnected from %1", details.description()), true);
+                    emit error(tr("Failed to send command. Disconnected from %1").arg(details.description()), true);
                 }
             }
         }
     }
     DBUG << (void *)(&sock) << "sendCommand - sent";
     if (QAbstractSocket::ConnectedState==sock.state()) {
-        connTimer->start(30000);
+        connTimer->start(constConnTimer);
     } else {
         connTimer->stop();
     }
@@ -766,10 +780,10 @@ void MPDConnection::add(const QStringList &origList, quint32 pos, quint32 size,
     }
 
     QList<QStringList> fileLists;
-    if (priority.count()<=1 && files.count()>maxFilesPerAddCommand) {
-        int numChunks=(files.count()/maxFilesPerAddCommand)+(files.count()%maxFilesPerAddCommand ? 1 : 0);
+    if (priority.count()<=1 && files.count()>constMaxFilesPerAddCommand) {
+        int numChunks=(files.count()/constMaxFilesPerAddCommand)+(files.count()%constMaxFilesPerAddCommand ? 1 : 0);
         for (int i=0; i<numChunks; ++i) {
-            fileLists.append(files.mid(i*maxFilesPerAddCommand, maxFilesPerAddCommand));
+            fileLists.append(files.mid(i*constMaxFilesPerAddCommand, constMaxFilesPerAddCommand));
         }
     } else {
         fileLists.append(files);
@@ -931,7 +945,7 @@ void MPDConnection::move(const QList<quint32> &items, quint32 pos, quint32 size)
 
 void MPDConnection::setOrder(const QList<quint32> &items)
 {
-    QByteArray cmd = /*name.isEmpty() ? */"move " /*: ("playlistmove "+encodeName(name)+" ")*/;
+    QByteArray cmd("move ");
     QByteArray send;
     QList<qint32> positions;
     quint32 numChanges=0;
@@ -1157,6 +1171,8 @@ void MPDConnection::setCrossFade(int secs)
 void MPDConnection::setReplayGain(const QString &v)
 {
     if (replaygainSupported()) {
+        // Issue #1041 - MPD does not seem to persist user/client made replaygain changes, so store in Cantata's config file.
+        Settings::self()->saveReplayGain(details.name, v);
         sendCommand("replay_gain_mode "+v.toLatin1());
     }
 }
@@ -1167,7 +1183,10 @@ void MPDConnection::getReplayGain()
         QStringList lines=QString(sendCommand("replay_gain_status").data).split('\n', QString::SkipEmptyParts);
 
         if (2==lines.count() && "OK"==lines[1] && lines[0].startsWith(QLatin1String("replay_gain_mode: "))) {
-            emit replayGain(lines[0].mid(18));
+            QString mode=lines[0].mid(18);
+            // Issue #1041 - MPD does not seem to persist user/client made replaygain changes, so store in Cantata's config file.
+            Settings::self()->saveReplayGain(details.name, mode);
+            emit replayGain(mode);
         } else {
             emit replayGain(QString());
         }
@@ -1205,20 +1224,31 @@ void MPDConnection::playFirstTrack(bool emitErrors)
     sendCommand("play 0", emitErrors);
 }
 
-void MPDConnection::seek(bool fwd)
+void MPDConnection::seek()
 {
+    QObject *s=sender();
+    int offset=s ? s->property("offset").toInt() : 0;
+    if (0==offset) {
+        return;
+    }
     toggleStopAfterCurrent(false);
     Response response=sendCommand("status");
     if (response.ok) {
         MPDStatusValues sv=MPDParseUtils::parseStatus(response.data);
-        if (fwd) {
-            if (sv.timeElapsed+seekStep<sv.timeTotal) {
-                setSeek(sv.song, sv.timeElapsed+seekStep);
+        if (offset>0) {
+            if (sv.timeElapsed+offset<sv.timeTotal) {
+                setSeek(sv.song, sv.timeElapsed+offset);
             } else {
                 goToNext();
             }
         } else {
-            setSeek(sv.song, sv.timeElapsed>=seekStep ? sv.timeElapsed-seekStep : 0);
+            if (sv.timeElapsed+offset>=0) {
+                setSeek(sv.song, sv.timeElapsed+offset);
+            } else {
+                // Not sure about this!!!
+                /*goToPrevious();*/
+                setSeek(sv.song, 0);
+            }
         }
     }
 }
@@ -1342,13 +1372,17 @@ void MPDConnection::getStats()
         dbUpdate=stats.dbUpdate;
         mopidy=0==stats.artists && 0==stats.albums && 0==stats.songs &&
                0==stats.uptime && 0==stats.playtime && 0==stats.dbPlaytime && 0==dbUpdate;
+        if (mopidy) {
+            // Set version to 1 so that SQL cache is updated - it uses 0 as intial value
+            dbUpdate=stats.dbUpdate=1;
+        }
         emit statsUpdated(stats);
     }
 }
 
 void MPDConnection::getStatus()
 {
-    Response response=sendCommand("status");
+   Response response=sendCommand("status");
     if (response.ok) {
         MPDStatusValues sv=MPDParseUtils::parseStatus(response.data);
         lastStatusPlayQueueVersion=sv.playlist;
@@ -1371,6 +1405,11 @@ void MPDConnection::getStatus()
             emit updatedDatabase();
         }
         emitStatusUpdated(sv);
+
+        // If playlist length does not match number of IDs, then refresh
+        if (sv.playlistLength!=playQueueIds.length()) {
+            playListInfo();
+        }
     }
 }
 
@@ -1490,16 +1529,13 @@ void MPDConnection::parseIdleReturn(const QByteArray &data)
                 outputs();
             } else if (constIdleStickerValue==value) {
                 emit stickerDbChanged();
-            }
-            #ifndef CANTATA_WEB
-            else if (constIdleSubscriptionValue==value) {
+            } else if (constIdleSubscriptionValue==value) {
                 //if (dynamicId.isEmpty()) {
                     setupRemoteDynamic();
                 //}
             } else if (constIdleMessageValue==value) {
                 readRemoteDynamicMessages();
             }
-            #endif
         }
     }
 
@@ -1528,13 +1564,12 @@ void MPDConnection::enableOutput(int id, bool enable)
  */
 void MPDConnection::update()
 {
-    if (sendCommand("update").ok) {
-        if (isMopdidy()) {
-            // Mopidy does not support MPD's update command. So, when user presses update DB, what we
-            // do instead is clear library/dir caches, then when response to getStats is received,
-            // library/dir should get refreshed...
-            getStats();
-        }
+    if (mopidy) {
+        // Mopidy does not support MPD's update command. So, when user presses update DB, what we
+        // just reload the library.
+        loadLibrary();
+    } else {
+        sendCommand("update");
     }
 }
 
@@ -1613,7 +1648,7 @@ void MPDConnection::renamePlaylist(const QString oldName, const QString newName)
     if (sendCommand("rename "+encodeName(oldName)+' '+encodeName(newName), false).ok) {
         emit playlistRenamed(oldName, newName);
     } else {
-        emit error(i18n("Failed to rename <b>%1</b> to <b>%2</b>", oldName, newName));
+        emit error(tr("Failed to rename <b>%1</b> to <b>%2</b>").arg(oldName).arg(newName));
     }
 }
 
@@ -1625,7 +1660,7 @@ void MPDConnection::removePlaylist(const QString &name)
 void MPDConnection::savePlaylist(const QString &name)
 {
     if (!sendCommand("save "+encodeName(name), false).ok) {
-        emit error(i18n("Failed to save <b>%1</b>", name));
+        emit error(tr("Failed to save <b>%1</b>").arg(name));
     }
 }
 
@@ -1638,10 +1673,10 @@ void MPDConnection::addToPlaylist(const QString &name, const QStringList &songs,
     if (!name.isEmpty()) {
         foreach (const QString &song, songs) {
             if (CueFile::isCue(song)) {
-                emit error(i18n("You cannot add parts of a cue sheet to a playlist!")+QLatin1String(" (")+song+QLatin1Char(')'));
+                emit error(tr("You cannot add parts of a cue sheet to a playlist!")+QLatin1String(" (")+song+QLatin1Char(')'));
                 return;
             } else if (isPlaylist(song)) {
-                emit error(i18n("You cannot add a playlist to another playlist!")+QLatin1String(" (")+song+QLatin1Char(')'));
+                emit error(tr("You cannot add a playlist to another playlist!")+QLatin1String(" (")+song+QLatin1Char(')'));
                 return;
             }
         }
@@ -1819,14 +1854,13 @@ void MPDConnection::editStream(const QString &url, const QString &name, quint32
 void MPDConnection::sendClientMessage(const QString &channel, const QString &msg, const QString &clientName)
 {
     if (!sendCommand("sendmessage "+channel.toUtf8()+" "+msg.toUtf8(), false).ok) {
-        emit error(i18n("Failed to send '%1' to %2. Please check %2 is registered with MPD.", msg, clientName.isEmpty() ? channel : clientName));
+        emit error(tr("Failed to send '%1' to %2. Please check %2 is registered with MPD.").arg(msg).arg(clientName.isEmpty() ? channel : clientName));
         emit clientMessageFailed(channel, msg);
     }
 }
 
 void MPDConnection::sendDynamicMessage(const QStringList &msg)
 {
-    #ifndef CANTATA_WEB
     // Check whether cantata-dynamic is still alive, by seeing if its channel is still open...
     if (1==msg.count() && QLatin1String("ping")==msg.at(0)) {
         Response response=sendCommand("channels");
@@ -1854,9 +1888,6 @@ void MPDConnection::sendDynamicMessage(const QStringList &msg)
     if (!sendCommand("sendmessage "+constDynamicIn+" "+data).ok) {
         emit dynamicSupport(false);
     }
-    #else
-    Q_UNUSED(msg)
-    #endif
 }
 
 void MPDConnection::moveInPlaylist(const QString &name, const QList<quint32> &items, quint32 pos, quint32 size)
@@ -1924,12 +1955,12 @@ bool MPDConnection::recursivelyListDir(const QString &dir, QList<Song> &songs)
 {
     bool topLevel="/"==dir || ""==dir;
 
-    // UPnP database backend does not list separate metadata items, so if "list genre" returns
-    // empty response assume this is a UPnP backend and dont attempt to get rest of data...
-    // Although we dont use "list XXX", lsinfo will return duplciate items (due to the way most
-    // UPnP servers returing directories of classifications - Genre/Album/Tracks, Artist/Album/Tracks,
-    // etc...
-    if (topLevel) {
+    if (topLevel && !mopidy) {
+        // UPnP database backend does not list separate metadata items, so if "list genre" returns
+        // empty response assume this is a UPnP backend and dont attempt to get rest of data...
+        // Although we dont use "list XXX", lsinfo will return duplciate items (due to the way most
+        // UPnP servers returing directories of classifications - Genre/Album/Tracks, Artist/Album/Tracks,
+        // etc...
         Response response=sendCommand("list genre", false, false);
         if (!response.ok || response.data.split('\n').length()<3) { // 2 lines - OK and blank
             // ..just to be 100% sure, check no artists either...
@@ -1940,7 +1971,9 @@ bool MPDConnection::recursivelyListDir(const QString &dir, QList<Song> &songs)
         }
     }
 
-    Response response=sendCommand(topLevel ? "lsinfo" : ("lsinfo "+encodeName(dir)));
+    Response response=sendCommand(topLevel
+                                    ? QByteArray(mopidy ? "lsinfo \"Local media\"" : "lsinfo")
+                                    : ("lsinfo "+encodeName(dir)));
     if (response.ok) {
         QStringList subDirs;
         MPDParseUtils::parseDirItems(response.data, details.dir, ver, songs, dir, subDirs, MPDParseUtils::Loc_Library);
@@ -1991,7 +2024,9 @@ QStringList MPDConnection::getAllFiles(const QString &dir)
         QList<Song> songs;
         MPDParseUtils::parseDirItems(response.data, details.dir, ver, songs, dir, subDirs, MPDParseUtils::Loc_Browse);
         foreach (const Song &song, songs) {
-            files.append(song.file);
+            if (Song::Playlist!=song.type) {
+                files.append(song.file);
+            }
         }
         foreach (const QString &sub, subDirs) {
             files+=getAllFiles(sub);
@@ -2001,13 +2036,12 @@ QStringList MPDConnection::getAllFiles(const QString &dir)
     return files;
 }
 
-#ifndef CANTATA_WEB
 bool MPDConnection::checkRemoteDynamicSupport()
 {
     if (ver>=CANTATA_MAKE_VERSION(0,17,0)) {
         Response response;
         if (-1!=idleSocket.write("channels\n")) {
-            idleSocket.waitForBytesWritten(socketTimeout(9));
+            idleSocket.waitForBytesWritten(constSocketCommsTimeout);
             response=readReply(idleSocket);
             if (response.ok) {
                 return MPDParseUtils::parseList(response.data, QByteArray("channel: ")).toSet().contains(constDynamicIn);
@@ -2020,7 +2054,7 @@ bool MPDConnection::checkRemoteDynamicSupport()
 bool MPDConnection::subscribe(const QByteArray &channel)
 {
     if (-1!=idleSocket.write("subscribe \""+channel+"\"\n")) {
-        idleSocket.waitForBytesWritten(socketTimeout(128));
+        idleSocket.waitForBytesWritten(constSocketCommsTimeout);
         Response response=readReply(idleSocket);
         if (response.ok || response.data.startsWith("ACK [56@0]")) { // ACK => already subscribed...
             DBUG << "Created subscription to " << channel;
@@ -2055,7 +2089,7 @@ void MPDConnection::setupRemoteDynamic()
 void MPDConnection::readRemoteDynamicMessages()
 {
     if (-1!=idleSocket.write("readmessages\n")) {
-        idleSocket.waitForBytesWritten(socketTimeout(22));
+        idleSocket.waitForBytesWritten(constSocketCommsTimeout);
         Response response=readReply(idleSocket);
         if (response.ok) {
             MPDParseUtils::MessageMap messages=MPDParseUtils::parseMessages(response.data);
@@ -2086,8 +2120,6 @@ void MPDConnection::readRemoteDynamicMessages()
     }
 }
 
-#endif
-
 int MPDConnection::getVolume()
 {
     Response response=sendCommand("status");
@@ -2105,7 +2137,7 @@ void MPDConnection::setRating(const QString &file, quint8 val)
     }
 
     if (!canUseStickers) {
-        emit error(i18n("Cannot store ratings, as the 'sticker' MPD command is not supported."));
+        emit error(tr("Cannot store ratings, as the 'sticker' MPD command is not supported."));
         return;
     }
 
@@ -2132,15 +2164,15 @@ void MPDConnection::setRating(const QStringList &files, quint8 val)
     }
 
     if (!canUseStickers) {
-        emit error(i18n("Cannot store ratings, as the 'sticker' MPD command is not supported."));
+        emit error(tr("Cannot store ratings, as the 'sticker' MPD command is not supported."));
         return;
     }
 
     QList<QStringList> fileLists;
-    if (files.count()>maxFilesPerAddCommand) {
-        int numChunks=(files.count()/maxFilesPerAddCommand)+(files.count()%maxFilesPerAddCommand ? 1 : 0);
+    if (files.count()>constMaxFilesPerAddCommand) {
+        int numChunks=(files.count()/constMaxFilesPerAddCommand)+(files.count()%constMaxFilesPerAddCommand ? 1 : 0);
         for (int i=0; i<numChunks; ++i) {
-            fileLists.append(files.mid(i*maxFilesPerAddCommand, maxFilesPerAddCommand));
+            fileLists.append(files.mid(i*constMaxFilesPerAddCommand, constMaxFilesPerAddCommand));
         }
     } else {
         fileLists.append(files);
