@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -27,8 +27,7 @@
 #include <cmath>
 #include "config.h"
 #include "song.h"
-#include "support/localize.h"
-#if !defined CANTATA_NO_UI_FUNCTIONS && !defined CANTATA_WEB
+#if !defined CANTATA_NO_UI_FUNCTIONS
 #include "online/onlineservice.h"
 #endif
 #include <QStringList>
@@ -57,8 +56,8 @@ const QString & Song::variousArtists()
 
 void Song::initTranslations()
 {
-    unknownStr=i18n("Unknown");
-    variousArtistsStr=i18n("Various Artists");
+    unknownStr=QObject::tr("Unknown");
+    variousArtistsStr=QObject::tr("Various Artists");
 }
 
 // When displaying albums, we use the 1st track's year as the year of the album.
@@ -147,10 +146,12 @@ void Song::setComposerGenres(const QSet<QString> &g)
 
 Song::Song()
     : extraFields(0)
-    , disc(0)
     , priority(0)
+    , disc(0)
+    , blank(0)
     , time(0)
     , track(0)
+    , origYear(0)
     , year(0)
     , type(Standard)
     , guessed(false)
@@ -174,9 +175,13 @@ Song & Song::operator=(const Song &s)
     track = s.track;
 //     pos = s.pos;
     disc = s.disc;
+    blank = s.blank;
     priority = s.priority;
+    origYear = s.origYear;
     year = s.year;
-    genre = s.genre;
+    for (int i=0; i<constNumGenres; ++i) {
+        genres[i]=s.genres[i];
+    }
     size = s.size;
     rating = s.rating;
     key = s.key;
@@ -238,6 +243,9 @@ int Song::compareTo(const Song &o) const
         if (year!=o.year) {
             return year<o.year ? -1 : 1;
         }
+        if (origYear!=o.origYear) {
+            return origYear<o.origYear ? -1 : 1;
+        }
         compare=title.localeAwareCompare(o.title);
         if (0!=compare) {
             return compare;
@@ -246,7 +254,7 @@ int Song::compareTo(const Song &o) const
         if (0!=compare) {
             return compare;
         }
-        compare=genre.compare(o.genre);
+        compare=compareGenres(o);
         if (0!=compare) {
             return compare;
         }
@@ -322,15 +330,18 @@ void Song::fillEmptyFields()
 {
     if (artist.isEmpty()) {
         artist = unknownStr;
+        blank |= BlankArtist;
     }
     if (album.isEmpty()) {
         album = unknownStr;
+        blank |= BlankAlbum;
     }
     if (title.isEmpty()) {
         title = unknownStr;
+        blank |= BlankTitle;
     }
-    if (genre.isEmpty()) {
-        genre = unknownStr;
+    if (genres[0].isEmpty()) {
+        genres[0]=unknownStr;
     }
 }
 
@@ -350,25 +361,11 @@ void Song::clearKeyStore(int location)
 
 QString Song::displayAlbum(const QString &albumName, quint16 albumYear)
 {
-    return albumYear>0 ? albumName+QLatin1String(" (")+QString::number(albumYear)+QLatin1Char(')') : albumName;
-}
-
-QString Song::combineGenres(const QSet<QString> &genres)
-{
-    if (1==genres.size()) {
-        return *(genres.begin());
+    QString d=albumYear>0 ? albumName+QLatin1String(" (")+QString::number(albumYear)+QLatin1Char(')') : albumName;
+    while (d.contains(") (")) {
+        d=d.replace(") (", ", ");
     }
-
-    QStringList list=genres.toList();
-    qSort(list);
-    QString g;
-    foreach (const QString &e, list) {
-        if (!g.isEmpty()) {
-            g+=constGenreSep;
-        }
-        g+=e;
-    }
-    return g;
+    return d;
 }
 
 static QSet<QString> prefixesToIngore=QSet<QString>() << QLatin1String("The");
@@ -440,38 +437,26 @@ void Song::clear()
     track = 0;
 //     pos = 0;
     disc = 0;
+    blank = 0;
     year = 0;
-    genre.clear();
+    origYear = 0;
+    for (int i=0; i<constNumGenres; ++i) {
+        genres[i]=QString();
+    }
     size = 0;
     extra.clear();
     type = Standard;
 }
 
-const QLatin1Char Song::constGenreSep(',');
 const QLatin1Char Song::constFieldSep('\001');
 
 void Song::addGenre(const QString &g)
 {
-    if (g.isEmpty()) {
-        return;
-    }
-    if (!genre.isEmpty()) {
-        genre+=constGenreSep;
-    }
-    genre+=g.trimmed();
-}
-
-QStringList Song::genres() const
-{
-    return genre.split(constGenreSep, QString::SkipEmptyParts);
-}
-
-void Song::orderGenres()
-{
-    QStringList g=genres();
-    if (g.count()>1) {
-        qSort(g);
-        genre=g.join(QString()+constGenreSep);
+    for (int i=0; i<constNumGenres; ++i) {
+        if (genres[i].isEmpty()) {
+            genres[i]=g;
+            break;
+        }
     }
 }
 
@@ -486,21 +471,20 @@ QString Song::entryName() const
 
 QString Song::artistOrComposer() const
 {
-    if (!compGenres.isEmpty()) {
+    if (useComposer()) {
         QString c=composer();
-        if (compGenres.contains(genre) && !c.isEmpty()) {
+        if (!c.isEmpty()) {
             return c;
         }
     }
-
     return albumArtist();
 }
 
 QString Song::albumName() const
 {
-    if (!compGenres.isEmpty()) {
+    if (useComposer()) {
         QString c=composer();
-        if (compGenres.contains(genre) && !c.isEmpty() && c!=albumArtist()) {
+        if (!c.isEmpty() && c!=albumArtist()) {
             return album+QLatin1String(" (")+albumArtist()+QLatin1Char(')');
         }
     }
@@ -521,7 +505,7 @@ QString Song::artistSong() const
 
 QString Song::trackAndTitleStr(bool showArtistIfDifferent) const
 {
-    #if !defined CANTATA_NO_UI_FUNCTIONS && !defined CANTATA_WEB
+    #if !defined CANTATA_NO_UI_FUNCTIONS
     if ((OnlineSvrTrack==type || Song::CantataStream) && OnlineService::showLogoAsCover(*this)) {
         return artistSong();
     }
@@ -533,7 +517,8 @@ QString Song::trackAndTitleStr(bool showArtistIfDifferent) const
 //    }
     return //(disc>0 ? (QString::number(disc)+QLatin1Char('.')) : QString())+
            (track>0 ? (track>9 ? QString::number(track)+QLatin1Char(' ') : (QLatin1Char('0')+QString::number(track)+QLatin1Char(' '))) : QString())+
-           (showArtistIfDifferent && diffArtist() ? artistSong() : title);
+           (showArtistIfDifferent && diffArtist() ? artistSong() : title) +
+           (origYear>0 && origYear != year ? QLatin1String(" (")+QString::number(origYear)+QLatin1Char(')') : QString());
 }
 
 #ifndef CANTATA_NO_UI_FUNCTIONS
@@ -551,26 +536,29 @@ QString Song::toolTip() const
     return QString();
     #else
     QString toolTip=QLatin1String("<table>");
-    addField(i18n("Title"), title, toolTip);
-    addField(i18n("Artist"), artist, toolTip);
+    addField(QObject::tr("Title"), title, toolTip);
+    addField(QObject::tr("Artist"), artist, toolTip);
     if (albumartist!=artist) {
-        addField(i18n("Album artist"), albumartist, toolTip);
+        addField(QObject::tr("Album artist"), albumartist, toolTip);
     }
-    addField(i18n("Composer"), composer(), toolTip);
-    addField(i18n("Performer"), performer(), toolTip);
-    addField(i18n("Album"), album, toolTip);
+    addField(QObject::tr("Composer"), composer(), toolTip);
+    addField(QObject::tr("Performer"), performer(), toolTip);
+    addField(QObject::tr("Album"), album, toolTip);
     if (track>0) {
-        addField(i18n("Track number"), QString::number(track), toolTip);
+        addField(QObject::tr("Track number"), QString::number(track), toolTip);
     }
     if (disc>0) {
-        addField(i18n("Disc number"), QString::number(disc), toolTip);
+        addField(QObject::tr("Disc number"), QString::number(disc), toolTip);
     }
-    addField(i18n("Genre"), displayGenre(), toolTip);
+    addField(QObject::tr("Genre"), displayGenre(), toolTip);
     if (year>0) {
-        addField(i18n("Year"), QString::number(year), toolTip);
+        addField(QObject::tr("Year"), QString::number(year), toolTip);
+    }
+    if (origYear>0) {
+        addField(QObject::tr("Orignal Year"), QString::number(origYear), toolTip);
     }
     if (time>0) {
-        addField(i18n("Length"), Utils::formatTime(time, true), toolTip);
+        addField(QObject::tr("Length"), Utils::formatTime(time, true), toolTip);
     }
     toolTip+=QLatin1String("</table>");
 
@@ -581,6 +569,26 @@ QString Song::toolTip() const
     #endif
 }
 
+QString Song::displayGenre() const
+{
+    QString g=genres[0];
+    for (int i=1; i<constNumGenres&& !genres[i].isEmpty(); ++i) {
+        g+=QLatin1String(", ")+genres[i];
+    }
+    return g;
+}
+
+int Song::compareGenres(const Song &o) const
+{
+    for (int i=0; i<constNumGenres; ++i) {
+        int compare=genres[i].compare(o.genres[i]);
+        if (0!=compare) {
+            return compare;
+        }
+    }
+    return 0;
+}
+
 void Song::setExtraField(quint16 f, const QString &v)
 {
     if (v.isEmpty()) {
@@ -646,10 +654,12 @@ QString Song::capitalize(const QString &s)
     for (int i = 0; i < words.count(); i++) {
         QString word = words[i]; //.toLower();
         int j = 0;
-        while ( ('('==word[j] || '['==word[j] || '{'==word[j]) && j < word.length()) {
+        while ( j < word.length() && ('('==word[j] || '['==word[j] || '{'==word[j]) ) {
             j++;
         }
-        word[j] = word[j].toUpper();
+        if (j < word.length()) {
+            word[j] = word[j].toUpper();
+        }
         words[i] = word;
     }
     return words.join(" ");
@@ -683,7 +693,7 @@ bool Song::capitalise()
 
 QString Song::albumKey() const
 {
-    #if !defined CANTATA_NO_UI_FUNCTIONS && !defined CANTATA_WEB
+    #if !defined CANTATA_NO_UI_FUNCTIONS
     if ((OnlineSvrTrack==type || Song::CantataStream) && OnlineService::showLogoAsCover(*this)) {
         return onlineService();
     }
@@ -719,8 +729,8 @@ QString Song::describe(bool withMarkup) const
         return withMarkup ? albumText : (QLatin1String("<b>")+albumText+QLatin1String("</b>"));
     }
     QString descr=artist.isEmpty()
-                    ? i18nc("Song on Album", "<b>%1</b> on <b>%2</b>", title, albumText)
-                    : i18nc("Song by Artist on Album", "<b>%1</b> by <b>%2</b> on <b>%3</b>", title, artist, albumText);
+                    ? QObject::tr("<b>%1</b> on <b>%2</b>", "Song on Album").arg(title).arg(albumText)
+                    : QObject::tr("<b>%1</b> by <b>%2</b> on <b>%3</b>", "Song by Artist on Album").arg(title).arg(artist).arg(albumText);
 
     if (!withMarkup) {
         descr=descr.replace("<b>", "");
@@ -731,8 +741,12 @@ QString Song::describe(bool withMarkup) const
 
 bool Song::useComposer() const
 {
-    foreach (const QString &g, genres()) {
-        if (isComposerGenre(g)) {
+    if (compGenres.isEmpty()) {
+        return false;
+    }
+
+    for (int i=0; i<constNumGenres && !genres[i].isEmpty(); ++i) {
+        if (compGenres.contains(genres[i])) {
             return true;
         }
     }
@@ -769,15 +783,17 @@ void Song::populateSorts()
 //                ? QString()
 //                : artist.isEmpty()
 //                    ? title
-//                    : i18nc("track - artist", "%1 - %2", title, artist);
+//                    : trc("track - artist", "%1 - %2", title, artist);
 //}
 
-#ifdef ENABLE_EXTERNAL_TAGS
 QDataStream & operator<<(QDataStream &stream, const Song &song)
 {
     stream << song.id << song.file << song.album << song.artist << song.albumartist << song.title
-           << song.genre << song.disc << song.priority << song.time << song.track << (quint16)song.year
+           << song.disc << song.priority << song.time << song.track << (quint16)song.year // << song.origYear
            << (quint16)song.type << (bool)song.guessed << song.size << song.extra << song.extraFields;
+    for (int i=0; i<Song::constNumGenres; ++i) {
+        stream << song.genres[i];
+    }
     return stream;
 }
 
@@ -785,13 +801,18 @@ QDataStream & operator>>(QDataStream &stream, Song &song)
 {
     quint16 type;
     quint16 year;
+    quint8 disc;
     bool guessed;
     stream >> song.id >> song.file >> song.album >> song.artist >> song.albumartist >> song.title
-           >> song.genre >> song.disc >> song.priority >> song.time >> song.track >> year
+           >> disc >> song.priority >> song.time >> song.track >> year // >> song.origYear
            >> type >> guessed >> song.size >> song.extra >> song.extraFields;
     song.type=(Song::Type)type;
     song.year=year;
     song.guessed=guessed;
+    song.disc=disc;
+    for (int i=0; i<Song::constNumGenres; ++i) {
+        stream >> song.genres[i];
+    }
+
     return stream;
 }
-#endif
