@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  */
 
 /**************************************************************************
@@ -34,26 +34,23 @@
 **************************************************************************/
 
 #include "fancytabwidget.h"
-#include "localize.h"
 #include "icon.h"
-#include "gtkstyle.h"
 #include "action.h"
 #include "utils.h"
+#include "config.h"
 #ifdef Q_OS_MAC
 #include "osxstyle.h"
 #endif
+#include <QStyleOption>
 #include <QHBoxLayout>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QSplitter>
 #include <QStackedWidget>
-#include <QStyleOption>
 #include <QToolButton>
 #include <QToolTip>
 #include <QVBoxLayout>
 #include <QApplication>
-#include <QAnimationGroup>
-#include <QPropertyAnimation>
 #include <QSignalMapper>
 #include <QMenu>
 
@@ -76,7 +73,7 @@ static int smallIconSize=16;
 void FancyTabWidget::setup()
 {
     largeIconSize=Icon::stdSize(Utils::scaleForDpi(32));
-    smallIconSize=Icon::stdSize(Utils::scaleForDpi(Utils::touchFriendly() ? 22 : 16));
+    smallIconSize=Icon::stdSize(Utils::scaleForDpi(16));
 }
 
 int FancyTabWidget::iconSize(bool large)
@@ -86,12 +83,13 @@ int FancyTabWidget::iconSize(bool large)
 
 static void drawIcon(const QIcon &icon, const QRect &r, QPainter *p, const QSize &iconSize, bool selected)
 {
-    QPixmap px = icon.pixmap(iconSize, selected ? QIcon::Selected : QIcon::Normal);
-    #if QT_VERSION >= 0x050100
-    QSize layoutSize = px.size() / px.devicePixelRatio();
+    #ifdef Q_OS_WIN
+    Q_UNUSED(selected);
+    QPixmap px = icon.pixmap(iconSize, QIcon::Normal);
     #else
-    QSize layoutSize = px.size();
+    QPixmap px = icon.pixmap(iconSize, selected ? QIcon::Selected : QIcon::Normal);
     #endif
+    QSize layoutSize = px.size() / px.DEVICE_PIXEL_RATIO();
     p->drawPixmap(r.x()+(r.width()-layoutSize.width())/2.0, r.y()+(r.height()-layoutSize.height())/2.0, layoutSize.width(), layoutSize.height(), px);
 }
 
@@ -115,18 +113,18 @@ int FancyTabProxyStyle::styleHint(StyleHint hint, const QStyleOption *option, co
 
 void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const
 {
+    const QStyleOptionTab *tabOpt = qstyleoption_cast<const QStyleOptionTab*>(option);
 
-    const QStyleOptionTabV3 *v3Opt = qstyleoption_cast<const QStyleOptionTabV3*>(option);
-
-    if (element != CE_TabBarTab || !v3Opt) {
+    if (element != CE_TabBarTab || !tabOpt) {
         QProxyStyle::drawControl(element, option, p, widget);
         return;
     }
 
-    const QRect rect = v3Opt->rect;
-    const bool selected = v3Opt->state  &State_Selected;
-    const bool verticalTabs = v3Opt->shape == QTabBar::RoundedWest;
-    const QString text = v3Opt->text;
+    const QRect rect = tabOpt->rect;
+    const bool selected = tabOpt->state&State_Selected;
+    const bool underMouse = tabOpt->state&State_MouseOver;
+    const bool verticalTabs = tabOpt->shape == QTabBar::RoundedWest;
+    const QString text = tabOpt->text;
 
     QTransform m;
     if (verticalTabs) {
@@ -138,101 +136,41 @@ void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption
 
     const QRect draw_rect(QPoint(0, 0), m.mapRect(rect).size());
 
-    if (!selected && GtkStyle::isActive()) {
-        p->fillRect(option->rect, option->palette.background());
-    }
-
     p->save();
     p->setTransform(m);
 
-    QRect iconRect(QPoint(8, 0), v3Opt->iconSize);
+    QRect iconRect(QPoint(8, 0), tabOpt->iconSize);
     QRect textRect(iconRect.topRight() + QPoint(4, 0), draw_rect.size());
     textRect.setRight(draw_rect.width());
     iconRect.translate(0, (draw_rect.height() - iconRect.height()) / 2);
 
-    QStyleOptionViewItemV4 styleOpt;
-    styleOpt.palette=option->palette;
-    styleOpt.rect=draw_rect;
-    if (QStyleOptionTab::Beginning==v3Opt->position) {
-        styleOpt.rect.adjust(0, 0, -1, 0);
-    }
-    styleOpt.state=option->state;
-    styleOpt.state&=~(QStyle::State_Selected|QStyle::State_MouseOver);
-    styleOpt.state|=QStyle::State_Selected|QStyle::State_Enabled;
-    styleOpt.viewItemPosition = QStyleOptionViewItemV4::OnlyOne;
-    styleOpt.showDecorationSelected=true;
-    bool drawBgnd=true;
-    int fader = 1;
+    if (selected || underMouse) {
+        #ifdef Q_OS_MAC
+        QColor col = OSXStyle::self()->viewPalette().highlight().color();
+        #else
+        QColor col = option->palette.highlight().color();
+        #endif
 
-    if (!selected && drawBgnd) {
-        const QString faderKey = "tab_" + text + "_fader";
-        const QString animationKey = "tab_" + text + "_animation";
-
-        const QString tab_hover = widget->property("tab_hover").toString();
-        fader=widget->property(faderKey.toUtf8().constData()).toInt();
-        QPropertyAnimation *animation = widget->property(animationKey.toUtf8().constData()).value<QPropertyAnimation*>();
-
-        if (!animation) {
-            QWidget* mut_widget = const_cast<QWidget*>(widget);
-            fader = 0;
-            mut_widget->setProperty(faderKey.toUtf8().constData(), fader);
-            animation = new QPropertyAnimation(mut_widget, faderKey.toUtf8(), mut_widget);
-            connect(animation, SIGNAL(valueChanged(QVariant)), mut_widget, SLOT(update()));
-            mut_widget->setProperty(animationKey.toUtf8().constData(), QVariant::fromValue(animation));
-        }
-
-        if (text == tab_hover) {
-            if (animation->state() != QAbstractAnimation::Running && fader != 40) {
-                animation->stop();
-                animation->setDuration(80);
-                animation->setEndValue(50);
-                animation->start();
-            }
-        } else {
-            if (animation->state() != QAbstractAnimation::Running && fader != 0) {
-                animation->stop();
-                animation->setDuration(160);
-                animation->setEndValue(0);
-                animation->start();
-            }
-        }
-
-        if (fader<1) {
-            drawBgnd=false;
-        } else {
-            QColor col(styleOpt.palette.highlight().color());
-            col.setAlpha(fader);
-            styleOpt.palette.setColor(styleOpt.palette.currentColorGroup(), QPalette::Highlight, col);
-        }
-    }
-
-    if (drawBgnd) {
-        if (!selected && GtkStyle::isActive()) {
-            GtkStyle::drawSelection(styleOpt, p, (fader*1.0)/150.0);
-        } else {
-            #ifdef Q_OS_MAC
-            OSXStyle::self()->drawSelection(styleOpt, p, selected ? 1.0 : (fader*1.0)/150.0);
-            #else
-            #ifdef Q_OS_WIN
-            if (QPalette::Active!=styleOpt.palette.currentColorGroup()) {
-                styleOpt.palette.setColor(QPalette::Highlight, styleOpt.palette.color(QPalette::Window).darker(110));
-            }
-            #endif
-            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &styleOpt, p, 0);
-            #endif
+        QPainterPath path = Utils::buildPath(draw_rect, 2.0);
+        if (!selected) {
+            col.setAlphaF(0.2);
         }
+        p->setRenderHint(QPainter::Antialiasing);
+        p->fillPath(path, col);
     }
 
     int textFlags = Qt::AlignTop | Qt::AlignVCenter;
     #ifdef Q_OS_MAC
     p->setPen(selected && option->state&State_Active
               ? OSXStyle::self()->viewPalette().highlightedText().color() : OSXStyle::self()->viewPalette().foreground().color());
+    #elif defined Q_OS_WIN
+    p->setPen(QApplication::palette().foreground().color());
     #else
     p->setPen(selected && option->state&State_Active
               ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
     #endif
 
-    drawIcon(v3Opt->icon, iconRect, p, v3Opt->iconSize,
+    drawIcon(tabOpt->icon, iconRect, p, tabOpt->iconSize,
              selected && option->state&State_Active);
 
     QString txt=text;
@@ -281,39 +219,10 @@ bool FancyTabProxyStyle::eventFilter(QObject* o, QEvent* e)
 }
 
 FancyTab::FancyTab(FancyTabBar* tabbar)
-    : QWidget(tabbar), tabbar(tabbar), faderValue(0)
+    : QWidget(tabbar), tabbar(tabbar), underMouse(false)
 {
-    #ifndef Q_OS_MAC
-    animator.setPropertyName("fader");
-    animator.setTargetObject(this);
-    #endif
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
-}
-
-void FancyTab::fadeIn()
-{
-    #ifndef Q_OS_MAC
-    animator.stop();
-    animator.setDuration(80);
-    animator.setEndValue(50);
-    animator.start();
-    #endif
-}
-
-void FancyTab::fadeOut()
-{
-    #ifndef Q_OS_MAC
-    animator.stop();
-    animator.setDuration(160);
-    animator.setEndValue(0);
-    animator.start();
-    #endif
-}
-
-void FancyTab::setFader(float value)
-{
-    faderValue = value;
-    tabbar->update();
+    setAttribute(Qt::WA_Hover, true);
 }
 
 FancyTabBar::FancyTabBar(QWidget *parent, bool text, int iSize, Pos pos)
@@ -365,7 +274,7 @@ QSize FancyTab::sizeHint() const
         int width = qMax(iconSize, qMin(3*iconSize, textWidth)) + spacing;
         return QSize(width, iconSize + spacing + fm.height() + padding);
     } else {
-        return QSize(iconSize + spacing + padding, iconSize + spacing);
+        return QSize(iconSize + spacing + padding, iconSize + spacing + padding);
     }
 }
 
@@ -389,28 +298,27 @@ void FancyTabBar::paintEvent(QPaintEvent *event)
 {
     Q_UNUSED(event)
     QPainter p(this);
-    bool gtkStyle=GtkStyle::isActive();
 
     for (int i = 0; i < count(); ++i) {
         if (i != currentIndex()) {
-            paintTab(&p, i, gtkStyle);
+            paintTab(&p, i);
         }
     }
 
     // paint active tab last, since it overlaps the neighbors
     if (currentIndex() != -1) {
-        paintTab(&p, currentIndex(), gtkStyle);
+        paintTab(&p, currentIndex());
     }
 }
 
 void FancyTab::enterEvent(QEvent*)
 {
-    fadeIn();
+    underMouse = true;
 }
 
 void FancyTab::leaveEvent(QEvent*)
 {
-    fadeOut();
+    underMouse = false;
 }
 
 QSize FancyTabBar::sizeHint() const
@@ -488,7 +396,7 @@ void FancyTabBar::addSpacer(int size)
                                                            new QSpacerItem(0, size, QSizePolicy::Fixed, QSizePolicy::Maximum));
 }
 
-void FancyTabBar::paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const
+void FancyTabBar::paintTab(QPainter *painter, int tabIndex) const
 {
     if (!validIndex(tabIndex)) {
         qWarning("invalid index");
@@ -498,41 +406,21 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const
 
     QRect rect = tabRect(tabIndex);
     bool selected = (tabIndex == currentIdx);
+    bool underMouse = tabs[tabIndex]->underMouse;
 
-    QStyleOptionViewItemV4 styleOpt;
-    styleOpt.initFrom(this);
-    styleOpt.state&=~(QStyle::State_Selected|QStyle::State_MouseOver);
-    styleOpt.state|=QStyle::State_Selected|QStyle::State_Enabled;
-    styleOpt.rect=rect;
-    styleOpt.viewItemPosition = QStyleOptionViewItemV4::OnlyOne;
-    styleOpt.showDecorationSelected=true;
-    bool drawBgnd=true;
+    if (selected || underMouse) {
+        #ifdef Q_OS_MAC
+        QColor col = OSXStyle::self()->viewPalette().highlight().color();
+        #else
+        QColor col = palette().highlight().color();
+        #endif
 
-    if (!selected && drawBgnd) {
-        int fader=int(tabs[tabIndex]->fader());
-        if (fader<1) {
-            drawBgnd=false;
-        } else  {
-            QColor col(styleOpt.palette.highlight().color());
-            col.setAlpha(fader);
-            styleOpt.palette.setColor(styleOpt.palette.currentColorGroup(), QPalette::Highlight, col);
-        }
-    }
-    if (drawBgnd) {
-        if (!selected && gtkStyle) {
-            GtkStyle::drawSelection(styleOpt, painter, tabs[tabIndex]->fader()/150.0);
-        } else {
-            #ifdef Q_OS_MAC
-            OSXStyle::self()->drawSelection(styleOpt, painter, selected ? 1.0 : tabs[tabIndex]->fader()/150.0);
-            #else
-            #ifdef Q_OS_WIN
-            if (QPalette::Active!=styleOpt.palette.currentColorGroup()) {
-                styleOpt.palette.setColor(QPalette::Highlight, styleOpt.palette.color(QPalette::Window).darker(110));
-            }
-            #endif
-            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &styleOpt, painter, 0);
-            #endif
+        QPainterPath path = Utils::buildPath(rect, 2.0);
+        if (!selected) {
+            col.setAlphaF(0.2);
         }
+        painter->setRenderHint(QPainter::Antialiasing);
+        painter->fillPath(path, col);
     }
 
     selected = selected && (palette().currentColorGroup()==QPalette::Active ||
@@ -547,6 +435,8 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const
 
         #ifdef Q_OS_MAC
         painter->setPen(selected ? OSXStyle::self()->viewPalette().highlightedText().color() : OSXStyle::self()->viewPalette().foreground().color());
+        #elif defined Q_OS_WIN
+        painter->setPen(QApplication::palette().foreground().color());
         #else
         painter->setPen(selected ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
         #endif
@@ -570,7 +460,7 @@ void FancyTabBar::setCurrentIndex(int index)
     emit currentChanged(currentIdx);
 }
 
-FancyTabWidget::FancyTabWidget(QWidget *parent, bool allowContextMenu)
+FancyTabWidget::FancyTabWidget(QWidget *parent)
     : QWidget(parent)
     , styleSetting(0)
     , tabBar(NULL)
@@ -580,7 +470,6 @@ FancyTabWidget::FancyTabWidget(QWidget *parent, bool allowContextMenu)
     , topLayout(new QVBoxLayout)
     , menu(0)
     , proxyStyle(new FancyTabProxyStyle)
-    , allowContext(allowContextMenu)
 {
     sideLayout->setSpacing(0);
     sideLayout->setMargin(0);
@@ -670,50 +559,6 @@ void FancyTabWidget::showWidget(int index)
     emit currentChanged(idx);
 }
 
-void FancyTabWidget::contextMenuEvent(QContextMenuEvent *e)
-{
-    if (!allowContext) {
-        return;
-    }
-
-    // Check we are over tab space...
-    if (Tab==(styleSetting&Style_Mask)) {
-        if (QApplication::widgetAt(e->globalPos())!=tabBar) {
-            return;
-        }
-    }
-    else {
-        switch (styleSetting&Position_Mask) {
-        case Bot:
-            if (e->pos().y()<=(sideWidget->pos().y()+(sideWidget->height()-tabBar->height()))) {
-                return;
-            }
-            break;
-        case Top:
-            if (e->pos().y()>(sideWidget->pos().y()+tabBar->height())) {
-                return;
-            }
-            break;
-        default:
-            if (QApplication::isRightToLeft()) {
-                if (e->pos().x()<=sideWidget->pos().x()) {
-                    return;
-                }
-            } else if (e->pos().x()>=sideWidget->rect().right()) {
-                return;
-            }
-        }
-    }
-
-    if (!menu) {
-        menu = new QMenu(this);
-        QAction *act=new QAction(i18n("Configure..."), this);
-        connect(act, SIGNAL(triggered()), SIGNAL(configRequested()));
-        menu->addAction(act);
-    }
-    menu->popup(e->globalPos());
-}
-
 void FancyTabWidget::setStyle(int s)
 {
     if (s==styleSetting && tabBar) {
