@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,11 +23,9 @@
 
 #include "utils.h"
 #include "config.h"
-#include "localize.h"
 #include <QFile>
 #include <QFileInfo>
 #include <QDir>
-#ifndef CANTATA_WEB
 #include <QProcess>
 #include <QApplication>
 #include <QDateTime>
@@ -36,15 +34,13 @@
 #include <QStyle>
 #include <QDesktopWidget>
 #include <QEventLoop>
-#if QT_VERSION >= 0x050000
 #include <QStandardPaths>
-#else
-#include <QDesktopServices>
-#endif
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KStandardDirs>
-#endif
+#ifndef _MSC_VER 
 #include <unistd.h>
+#include <utime.h>
+#else
+#include <sys/utime.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifndef Q_OS_WIN
@@ -52,8 +48,6 @@
 #include <pwd.h>
 #endif
 #include <sys/types.h>
-#include <utime.h>
-#endif
 
 const QLatin1Char Utils::constDirSep('/');
 const QLatin1String Utils::constDirSepStr("/");
@@ -176,7 +170,6 @@ bool Utils::isDirReadable(const QString &dir)
     #endif
 }
 
-#ifndef CANTATA_WEB
 QString Utils::strippedText(QString s)
 {
     s.remove(QString::fromLatin1("..."));
@@ -343,7 +336,6 @@ bool Utils::createWorldReadableDir(const QString &dir, const QString &base, cons
     #endif
 }
 
-#ifndef ENABLE_KDE_SUPPORT
 // Copied from KDE... START
 #include <QLocale>
 #include <fcntl.h>
@@ -435,19 +427,19 @@ QString Utils::formatByteSize(double size)
 
     if (useSiUnites) {
         switch(unit) {
-        case 0: return i18n("%1 B", size);
-        case 1: return i18n("%1 kB", locale.toString(size, 'f', 1));
-        case 2: return i18n("%1 MB", locale.toString(size, 'f', 1));
+        case 0: return QObject::tr("%1 B").arg(size);
+        case 1: return QObject::tr("%1 kB").arg(locale.toString(size, 'f', 1));
+        case 2: return QObject::tr("%1 MB").arg(locale.toString(size, 'f', 1));
         default:
-        case 3: return i18n("%1 GB", locale.toString(size, 'f', 1));
+        case 3: return QObject::tr("%1 GB").arg(locale.toString(size, 'f', 1));
         }
     } else {
         switch(unit) {
-        case 0: return i18n("%1 B", size);
-        case 1: return i18n("%1 KiB", locale.toString(size, 'f', 1));
-        case 2: return i18n("%1 MiB", locale.toString(size, 'f', 1));
+        case 0: return QObject::tr("%1 B").arg(size);
+        case 1: return QObject::tr("%1 KiB").arg(locale.toString(size, 'f', 1));
+        case 2: return QObject::tr("%1 MiB").arg(locale.toString(size, 'f', 1));
         default:
-        case 3: return i18n("%1 GiB", locale.toString(size, 'f', 1));
+        case 3: return QObject::tr("%1 GiB").arg(locale.toString(size, 'f', 1));
         }
     }
 }
@@ -629,8 +621,6 @@ QString Utils::findExe(const QString &appname, const QString &pstr)
 }
 // Copied from KDE... END
 
-#endif
-
 QString Utils::formatDuration(const quint32 totalseconds)
 {
     //Get the days,hours,minutes and seconds out of the total seconds
@@ -644,21 +634,15 @@ QString Utils::formatDuration(const quint32 totalseconds)
     //Convert hour,minutes and seconds to a QTime for easier parsing
     QTime time(hours, minutes, seconds);
 
-    #ifdef ENABLE_KDE_SUPPORT
-    return 0==days
-            ? time.toString("h:mm:ss")
-            : i18np("1 day %2", "%1 days %2", days, time.toString("h:mm:ss"));
-    #else
     return 0==days
             ? time.toString("h:mm:ss")
             : QString("%1:%2").arg(days).arg(time.toString("hh:mm:ss"));
-    #endif
 }
 
 QString Utils::formatTime(const quint32 seconds, bool zeroIsUnknown)
 {
     if (0==seconds && zeroIsUnknown) {
-        return i18n("Unknown");
+        return QObject::tr("Unknown");
     }
 
     static const quint32 constHour=60*60;
@@ -695,31 +679,19 @@ static QString userDir(const QString &mainDir, const QString &sub, bool create)
 
 QString Utils::dataDir(const QString &sub, bool create)
 {
-    #if defined Q_OS_WIN || defined Q_OS_MAC || defined ENABLE_UBUNTU
+    #if defined Q_OS_WIN || defined Q_OS_MAC
 
-    #if QT_VERSION >= 0x050000
     return userDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation)+constDirSep, sub, create);
-    #else
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation)+constDirSep, sub, create);
-    #endif
 
     #else
 
     static QString location;
     if (location.isEmpty()) {
-        #if QT_VERSION >= 0x050000
         location=QStandardPaths::writableLocation(QStandardPaths::DataLocation);
         if (QCoreApplication::organizationName()==QCoreApplication::applicationName()) {
             location=location.replace(QCoreApplication::organizationName()+Utils::constDirSep+QCoreApplication::applicationName(),
                                       QCoreApplication::applicationName());
         }
-        #else
-        location=QDesktopServices::storageLocation(QDesktopServices::DataLocation);
-        if (QCoreApplication::organizationName()==QCoreApplication::applicationName()) {
-            location=location.replace(QLatin1String("data")+Utils::constDirSep+QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
-                                      QCoreApplication::applicationName());
-        }
-        #endif
     }
     return userDir(location+constDirSep, sub, create);
 
@@ -728,23 +700,15 @@ QString Utils::dataDir(const QString &sub, bool create)
 
 QString Utils::cacheDir(const QString &sub, bool create)
 {
-    #if defined Q_OS_WIN || defined Q_OS_MAC || defined ENABLE_UBUNTU
+    #if defined Q_OS_WIN || defined Q_OS_MAC
 
-    #if QT_VERSION >= 0x050000
     return userDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)+constDirSep, sub, create);
-    #else
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::CacheLocation)+constDirSep, sub, create);
-    #endif
 
     #else
 
     static QString location;
     if (location.isEmpty()) {
-        #if QT_VERSION >= 0x050000
         location=QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
-        #else
-        location=QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
-        #endif
         if (QCoreApplication::organizationName()==QCoreApplication::applicationName()) {
             location=location.replace(QCoreApplication::organizationName()+Utils::constDirSep+QCoreApplication::applicationName(),
                                       QCoreApplication::applicationName());
@@ -773,7 +737,7 @@ QString Utils::helper(const QString &app)
     #elif defined Q_OS_MAC
     return fixPath(QCoreApplication::applicationDirPath())+app;
     #else
-    return QString(INSTALL_PREFIX "/"LINUX_LIB_DIR"/")+QCoreApplication::applicationName()+constDirSep+app;
+    return QString(INSTALL_PREFIX "/" LINUX_LIB_DIR "/")+QCoreApplication::applicationName()+constDirSep+app;
     #endif
 }
 
@@ -946,17 +910,6 @@ Utils::Desktop Utils::currentDe()
     return Other;
 }
 
-static bool isTouchFriendly=false;
-void Utils::setTouchFriendly(bool t)
-{
-    isTouchFriendly=t;
-}
-
-bool Utils::touchFriendly()
-{
-    return isTouchFriendly;
-}
-
 QPainterPath Utils::buildPath(const QRectF &r, double radius)
 {
     QPainterPath path;
@@ -970,15 +923,27 @@ QPainterPath Utils::buildPath(const QRectF &r, double radius)
     return path;
 }
 
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KWindowSystem>
-#endif
+QColor Utils::clampColor(const QColor &col)
+{
+    static const int constMin=64;
+    static const int constMax=240;
+
+    if (col.value()<constMin) {
+        return QColor(constMin, constMin, constMin);
+    } else if (col.value()>constMax) {
+        return QColor(constMax, constMax, constMax);
+    }
+    return col;
+}
+
+QColor Utils::monoIconColor()
+{
+    return clampColor(QApplication::palette().color(QPalette::Active, QPalette::WindowText));
+}
+
 #ifdef Q_OS_WIN
 // This is down here, because windows.h includes ALL windows stuff - and we get conflicts with MessageBox :-(
 #include <windows.h>
-#elif !defined Q_OS_MAC && QT_VERSION < 0x050000
-#include <QX11Info>
-#include <X11/Xlib.h>
 #endif
 
 void Utils::raiseWindow(QWidget *w)
@@ -990,7 +955,7 @@ void Utils::raiseWindow(QWidget *w)
     #ifdef Q_OS_WIN
     ::SetWindowPos(reinterpret_cast<HWND>(w->effectiveWinId()), HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
     ::SetWindowPos(reinterpret_cast<HWND>(w->effectiveWinId()), HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
-    #elif !defined Q_OS_WIN && QT_VERSION>=0x050000
+    #elif !defined Q_OS_WIN
     bool wasHidden=w->isHidden();
     #endif
 
@@ -1002,21 +967,6 @@ void Utils::raiseWindow(QWidget *w)
     #endif
     #if !defined Q_OS_WIN && !defined Q_OS_MAC
     // This section seems to be required for compiz, so that MPRIS.Raise actually shows the window, and not just highlight launcher.
-    #if QT_VERSION < 0x050000
-    static const Atom constNetActive=XInternAtom(QX11Info::display(), "_NET_ACTIVE_WINDOW", False);
-    QX11Info info;
-    XEvent xev;
-    xev.xclient.type = ClientMessage;
-    xev.xclient.serial = 0;
-    xev.xclient.send_event = True;
-    xev.xclient.message_type = constNetActive;
-    xev.xclient.display = QX11Info::display();
-    xev.xclient.window = w->effectiveWinId();
-    xev.xclient.format = 32;
-    xev.xclient.data.l[0] = 2;
-    xev.xclient.data.l[1] = xev.xclient.data.l[2] = xev.xclient.data.l[3] = xev.xclient.data.l[4] = 0;
-    XSendEvent(QX11Info::display(), QX11Info::appRootWindow(info.screen()), False, SubstructureRedirectMask|SubstructureNotifyMask, &xev);
-    #else // QT_VERSION < 0x050000
     QString wmctrl=Utils::findExe(QLatin1String("wmctrl"));
     if (!wmctrl.isEmpty()) {
         if (wasHidden) {
@@ -1024,11 +974,6 @@ void Utils::raiseWindow(QWidget *w)
         }
         QProcess::execute(wmctrl, QStringList() << QLatin1String("-i") << QLatin1String("-a") << QString::number(w->effectiveWinId()));
     }
-    #endif // QT_VERSION < 0x050000
-    #ifdef ENABLE_KDE_SUPPORT
-    KWindowSystem::forceActiveWindow(w->effectiveWinId());
-    #endif
     #endif
 }
 
-#endif // CANTATA_WEB
