@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,9 +28,7 @@
 #include "gui/settings.h"
 #include "mpd-interface/mpdconnection.h"
 #include "models/playqueuemodel.h"
-#include "support/squeezedtextlabel.h"
 #include "support/utils.h"
-#include "support/localize.h"
 #ifdef Q_OS_MAC
 #include "support/osxstyle.h"
 #endif
@@ -44,6 +42,8 @@
 #include <QToolTip>
 #include <QSpacerItem>
 
+static const int constPollMpd = 2; // Poll every 2 seconds when playing
+
 class PosSliderProxyStyle : public QProxyStyle
 {
 public:
@@ -132,7 +132,7 @@ public:
             break;
         case QEvent::HoverLeave:
             if (isEnabled()) {
-                setStyleSheet(QString());
+                setStyleSheet(QString("QLabel{color:%1;}").arg(((NowPlayingWidget *)parentWidget())->textColor().name()));
             }
         default:
             break;
@@ -256,7 +256,6 @@ NowPlayingWidget::NowPlayingWidget(QWidget *p)
     , timer(0)
     , lastVal(0)
     , pollCount(0)
-    , pollMpd(Settings::self()->mpdPoll())
 {
     track=new SqueezedTextLabel(this);
     artist=new SqueezedTextLabel(this);
@@ -297,9 +296,7 @@ NowPlayingWidget::NowPlayingWidget(QWidget *p)
     connect(slider, SIGNAL(sliderReleased()), this, SLOT(released()));
     connect(slider, SIGNAL(positionSet()), this, SIGNAL(sliderReleased()));
     connect(slider, SIGNAL(valueChanged(int)), this, SLOT(updateTimes()));
-    if (pollMpd>0) {
-        connect(this, SIGNAL(mpdPoll()), MPDConnection::self(), SLOT(getStatus()));
-    }
+    connect(this, SIGNAL(mpdPoll()), MPDConnection::self(), SLOT(getStatus()));
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     clearTimes();
     update(Song());
@@ -317,10 +314,10 @@ void NowPlayingWidget::update(const Song &song)
     if (song.isEmpty()) {
         track->setText(" ");
         artist->setText(" ");
-    } else if (song.isStream() && !song.isCantataStream() && !song.isCdda()) {
+    } else if (song.isStream() && !song.isCantataStream() && !song.isCdda() && !song.isDlnaStream()) {
         track->setText(name.isEmpty() ? Song::unknown() : name);
         if (song.artist.isEmpty() && song.title.isEmpty() && !name.isEmpty()) {
-            artist->setText(i18n("(Stream)"));
+            artist->setText(tr("(Stream)"));
         } else {
             artist->setText(song.artist.isEmpty() ? song.title : song.artistSong());
         }
@@ -328,14 +325,15 @@ void NowPlayingWidget::update(const Song &song)
         if (song.title.isEmpty() && song.artist.isEmpty() && (!name.isEmpty() || !song.file.isEmpty())) {
             track->setText(name.isEmpty() ? song.file : name);
         } else {
-            track->setText(song.title);
+            track->setText(song.title+(song.origYear>0 && song.origYear!=song.year ? QLatin1String(" (")+QString::number(song.origYear)+QLatin1Char(')') : QString()));
         }
         if (song.album.isEmpty() && song.artist.isEmpty()) {
             artist->setText(track->fullText().isEmpty() ? QString() : Song::unknown());
         } else if (song.album.isEmpty()) {
             artist->setText(song.artist);
         } else {
-            artist->setText(song.artist+QLatin1String(" - ")+song.displayAlbum());
+            // Artist here is always artist, and not album artist or composer
+            artist->setText(song.artist+QLatin1String(" - ")+song.displayAlbum(false));
         }
     }
 }
@@ -419,11 +417,9 @@ void NowPlayingWidget::updatePos()
     int elapsed=(startTime.elapsed()/1000.0)+0.5;
     slider->setValue(lastVal+elapsed);
     MPDStatus::self()->setGuessedElapsed(lastVal+elapsed);
-    if (pollMpd>0) {
-        if (++pollCount>=pollMpd) {
-            pollCount=0;
-            emit mpdPoll();
-        }
+    if (++pollCount>=constPollMpd) {
+        pollCount=0;
+        emit mpdPoll();
     }
 }
 
