@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2016 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2017 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -36,13 +36,11 @@ public:
     QSize sizeHint() const;
     void setMenu(QMenu *m);
     void paintEvent(QPaintEvent *e);
-    void setHideMenuIndicator(bool h) { hideMenuIndicator=h; }
     #ifdef UNITY_MENU_HACK
     void setIcon(const QIcon &i) { icon=i; }
     #endif
 
 private:
-    bool hideMenuIndicator;
     mutable QSize sh;
     #ifdef UNITY_MENU_HACK
     QIcon icon;
